#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from keystoneauth1 import loading as kaloading

from watcherclient import client as watcherclient
from watcherclient.common import httpclient
from watcherclient import exceptions
from watcherclient.tests.unit import utils


class ClientTest(utils.BaseTestCase):

    def test_get_client_with_auth_token_watcher_url(self):
        kwargs = {
            'watcher_url': 'http://watcher.example.org:9322/',
            'os_auth_token': 'USER_AUTH_TOKEN',
        }
        client = watcherclient.get_client('1', **kwargs)

        self.assertEqual('USER_AUTH_TOKEN', client.http_client.auth_token)
        self.assertEqual('http://watcher.example.org:9322/',
                         client.http_client.endpoint)

    @mock.patch.object(kaloading.session, 'Session', autospec=True)
    @mock.patch.object(kaloading, 'get_plugin_loader', autospec=True)
    def _test_get_client(self, mock_ks_loader, mock_ks_session,
                         version=None, auth='password', **kwargs):
        session = mock_ks_session.return_value.load_from_options.return_value
        session.get_endpoint.return_value = 'http://localhost:9322/v1/f14b4123'
        mock_ks_loader.return_value.load_from_options.return_value = 'auth'

        watcherclient.get_client('1', **kwargs)

        mock_ks_loader.assert_called_once_with(auth)
        mock_ks_session.return_value.load_from_options.assert_called_once_with(
            auth='auth', timeout=kwargs.get('timeout'),
            insecure=kwargs.get('insecure'), cert=kwargs.get('cert'),
            cacert=kwargs.get('cacert'), key=kwargs.get('key'))
        session.get_endpoint.assert_called_once_with(
            service_type=kwargs.get('os_service_type') or 'infra-optim',
            interface=kwargs.get('os_endpoint_type') or 'publicURL',
            region_name=kwargs.get('os_region_name'))

    def test_get_client_no_auth_token(self):
        kwargs = {
            'os_tenant_name': 'TENANT_NAME',
            'os_username': 'USERNAME',
            'os_password': 'PASSWORD',
            'os_auth_url': 'http://localhost:35357/v2.0',
            'os_auth_token': '',
        }
        self._test_get_client(**kwargs)

    def test_get_client_service_and_endpoint_type_defaults(self):
        kwargs = {
            'os_tenant_name': 'TENANT_NAME',
            'os_username': 'USERNAME',
            'os_password': 'PASSWORD',
            'os_auth_url': 'http://localhost:35357/v2.0',
            'os_auth_token': '',
            'os_service_type': '',
            'os_endpoint_type': ''
        }
        self._test_get_client(**kwargs)

    def test_get_client_with_region_no_auth_token(self):
        kwargs = {
            'os_tenant_name': 'TENANT_NAME',
            'os_username': 'USERNAME',
            'os_password': 'PASSWORD',
            'os_region_name': 'REGIONONE',
            'os_auth_url': 'http://localhost:35357/v2.0',
            'os_auth_token': '',
        }
        self._test_get_client(**kwargs)

    def test_get_client_no_url(self):
        kwargs = {
            'os_tenant_name': 'TENANT_NAME',
            'os_username': 'USERNAME',
            'os_password': 'PASSWORD',
            'os_auth_url': '',
        }
        self.assertRaises(
            exceptions.AmbiguousAuthSystem, watcherclient.get_client,
            '1', **kwargs)
        # test the alias as well to ensure backwards compatibility
        self.assertRaises(
            exceptions.AmbigiousAuthSystem, watcherclient.get_client,
            '1', **kwargs)

    def test_get_client_incorrect_auth_params(self):
        kwargs = {
            'os_tenant_name': 'TENANT_NAME',
            'os_username': 'USERNAME',
            'os_auth_url': 'http://localhost:35357/v2.0',
        }
        self.assertRaises(
            exceptions.AmbiguousAuthSystem, watcherclient.get_client,
            '1', **kwargs)

    def test_get_client_with_api_version_latest(self):
        kwargs = {
            'os_tenant_name': 'TENANT_NAME',
            'os_username': 'USERNAME',
            'os_password': 'PASSWORD',
            'os_auth_url': 'http://localhost:35357/v2.0',
            'os_auth_token': '',
            'os_infra_optim_api_version': "latest",
        }
        self._test_get_client(**kwargs)

    def test_get_client_with_api_version_numeric(self):
        kwargs = {
            'os_tenant_name': 'TENANT_NAME',
            'os_username': 'USERNAME',
            'os_password': 'PASSWORD',
            'os_auth_url': 'http://localhost:35357/v2.0',
            'os_auth_token': '',
            'os_infra_optim_api_version': "1.4",
        }
        self._test_get_client(**kwargs)

    def test_get_client_with_auth_token(self):
        kwargs = {
            'os_auth_url': 'http://localhost:35357/v2.0',
            'os_auth_token': 'USER_AUTH_TOKEN',
        }
        self._test_get_client(auth='token', **kwargs)

    def test_get_client_with_region_name_auth_token(self):
        kwargs = {
            'os_auth_url': 'http://localhost:35357/v2.0',
            'os_region_name': 'REGIONONE',
            'os_auth_token': 'USER_AUTH_TOKEN',
        }
        self._test_get_client(auth='token', **kwargs)

    def test_get_client_only_session_passed(self):
        session = mock.Mock()
        session.get_endpoint.return_value = 'http://localhost:35357/v2.0'
        kwargs = {
            'session': session,
        }
        watcherclient.get_client('1', **kwargs)
        session.get_endpoint.assert_called_once_with(
            service_type='infra-optim',
            interface='publicURL',
            region_name=None)

    def test_get_client_incorrect_session_passed(self):
        session = mock.Mock()
        session.get_endpoint.side_effect = Exception('boo')
        kwargs = {
            'session': session,
        }
        self.assertRaises(
            exceptions.AmbiguousAuthSystem, watcherclient.get_client,
            '1', **kwargs)

    @mock.patch.object(kaloading.session, 'Session', autospec=True)
    @mock.patch.object(kaloading, 'get_plugin_loader', autospec=True)
    def _test_loader_arguments_passed_correctly(
            self, mock_ks_loader, mock_ks_session,
            passed_kwargs, expected_kwargs):
        session = mock_ks_session.return_value.load_from_options.return_value
        session.get_endpoint.return_value = 'http://localhost:9322/v1/f14b4123'
        mock_ks_loader.return_value.load_from_options.return_value = 'auth'

        watcherclient.get_client('1', **passed_kwargs)

        mock_ks_loader.return_value.load_from_options.assert_called_once_with(
            **expected_kwargs)
        mock_ks_session.return_value.load_from_options.assert_called_once_with(
            auth='auth', timeout=passed_kwargs.get('timeout'),
            insecure=passed_kwargs.get('insecure'),
            cert=passed_kwargs.get('cert'),
            cacert=passed_kwargs.get('cacert'), key=passed_kwargs.get('key'))
        session.get_endpoint.assert_called_once_with(
            service_type=passed_kwargs.get('os_service_type') or 'infra-optim',
            interface=passed_kwargs.get('os_endpoint_type') or 'publicURL',
            region_name=passed_kwargs.get('os_region_name'))

    def test_loader_arguments_token(self):
        passed_kwargs = {
            'os_auth_url': 'http://localhost:35357/v3',
            'os_region_name': 'REGIONONE',
            'os_auth_token': 'USER_AUTH_TOKEN',
        }
        expected_kwargs = {
            'auth_url': 'http://localhost:35357/v3',
            'project_id': None,
            'project_name': None,
            'user_domain_id': None,
            'user_domain_name': None,
            'project_domain_id': None,
            'project_domain_name': None,
            'token': 'USER_AUTH_TOKEN'
        }
        self._test_loader_arguments_passed_correctly(
            passed_kwargs=passed_kwargs, expected_kwargs=expected_kwargs)

    def test_loader_arguments_password_tenant_name(self):
        passed_kwargs = {
            'os_auth_url': 'http://localhost:35357/v3',
            'os_region_name': 'REGIONONE',
            'os_tenant_name': 'TENANT',
            'os_username': 'user',
            'os_password': '1234',
            'os_project_domain_id': 'DEFAULT',
            'os_user_domain_id': 'DEFAULT'
        }
        expected_kwargs = {
            'auth_url': 'http://localhost:35357/v3',
            'project_id': None,
            'project_name': 'TENANT',
            'user_domain_id': 'DEFAULT',
            'user_domain_name': None,
            'project_domain_id': 'DEFAULT',
            'project_domain_name': None,
            'username': 'user',
            'password': '1234'
        }
        self._test_loader_arguments_passed_correctly(
            passed_kwargs=passed_kwargs, expected_kwargs=expected_kwargs)

    def test_loader_arguments_password_project_id(self):
        passed_kwargs = {
            'os_auth_url': 'http://localhost:35357/v3',
            'os_region_name': 'REGIONONE',
            'os_project_id': '1000',
            'os_username': 'user',
            'os_password': '1234',
            'os_project_domain_name': 'domain1',
            'os_user_domain_name': 'domain1'
        }
        expected_kwargs = {
            'auth_url': 'http://localhost:35357/v3',
            'project_id': '1000',
            'project_name': None,
            'user_domain_id': None,
            'user_domain_name': 'domain1',
            'project_domain_id': None,
            'project_domain_name': 'domain1',
            'username': 'user',
            'password': '1234'
        }
        self._test_loader_arguments_passed_correctly(
            passed_kwargs=passed_kwargs, expected_kwargs=expected_kwargs)

    @mock.patch.object(watcherclient, 'Client')
    @mock.patch.object(kaloading.session, 'Session', autospec=True)
    def test_correct_arguments_passed_to_client_constructor_noauth_mode(
            self, mock_ks_session, mock_client):
        kwargs = {
            'watcher_url': 'http://watcher.example.org:9322/',
            'os_auth_token': 'USER_AUTH_TOKEN',
            'os_infra_optim_api_version': 'latest',
            'insecure': True,
            'max_retries': 10,
            'retry_interval': 10,
            'os_cacert': 'data'
        }
        watcherclient.get_client('1', **kwargs)
        mock_client.assert_called_once_with(
            '1', 'http://watcher.example.org:9322/',
            **{
                'os_infra_optim_api_version': 'latest',
                'max_retries': 10,
                'retry_interval': 10,
                'token': 'USER_AUTH_TOKEN',
                'insecure': True,
                'ca_file': 'data',
                'cert_file': None,
                'key_file': None,
                'timeout': None,
                'session': None
            }
        )
        self.assertFalse(mock_ks_session.called)

    @mock.patch.object(watcherclient, 'Client')
    @mock.patch.object(kaloading.session, 'Session', autospec=True)
    def test_correct_arguments_passed_to_client_constructor_session_created(
            self, mock_ks_session, mock_client):
        session = mock_ks_session.return_value.load_from_options.return_value
        kwargs = {
            'os_auth_url': 'http://localhost:35357/v3',
            'os_region_name': 'REGIONONE',
            'os_project_id': '1000',
            'os_username': 'user',
            'os_password': '1234',
            'os_project_domain_name': 'domain1',
            'os_user_domain_name': 'domain1'
        }
        watcherclient.get_client('1', **kwargs)
        mock_client.assert_called_once_with(
            '1', session.get_endpoint.return_value,
            **{
                'os_infra_optim_api_version': None,
                'max_retries': None,
                'retry_interval': None,
                'session': session,
            }
        )

    @mock.patch.object(watcherclient, 'Client')
    @mock.patch.object(kaloading.session, 'Session', autospec=True)
    def test_correct_arguments_passed_to_client_constructor_session_passed(
            self, mock_ks_session, mock_client):
        session = mock.Mock()
        kwargs = {
            'session': session,
        }
        watcherclient.get_client('1', **kwargs)
        mock_client.assert_called_once_with(
            '1', session.get_endpoint.return_value,
            **{
                'os_infra_optim_api_version': None,
                'max_retries': None,
                'retry_interval': None,
                'session': session,
            }
        )
        self.assertFalse(mock_ks_session.called)

    def test_safe_header_with_auth_token(self):
        (name, value) = ('X-Auth-Token', u'3b640e2e64d946ac8f55615aff221dc1')
        expected_header = (u'X-Auth-Token',
                           '{SHA1}6de9fb3b0b89099030a54abfeb468e7b1b1f0f2b')
        client = httpclient.HTTPClient('http://localhost/')
        header_redact = client._process_header(name, value)
        self.assertEqual(expected_header, header_redact)

    def test_safe_header_with_no_auth_token(self):
        name, value = ('Accept', 'application/json')
        header = ('Accept', 'application/json')
        client = httpclient.HTTPClient('http://localhost/')
        header_redact = client._process_header(name, value)
        self.assertEqual(header, header_redact)

    def test_make_connection_url(self):
        endpoint = 'http://localhost/infra-optim'
        url = '/v1/goals'
        expected_url = 'http://localhost/infra-optim/v1/goals'
        client = httpclient.HTTPClient(endpoint)
        conn_url = client._make_connection_url(url)
        self.assertEqual(expected_url, conn_url)



# Copyright 2019 ZTE Corporation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from watcherclient._i18n import _
from watcherclient.common import command
from watcherclient import exceptions
from watcherclient.v1 import resource_fields as res_fields


class ListDataModel(command.Lister):
    """List information on retrieved data model."""

    def get_parser(self, prog_name):
        parser = super(ListDataModel, self).get_parser(prog_name)
        parser.add_argument(
            '--type',
            metavar='<type>',
            dest='type',
            help=_('Type of Datamodel user want to list. '
                   'Supported values: compute. '
                   'Future support values: storage, baremetal. '
                   'Default type is compute.'))
        parser.add_argument(
            '--audit',
            metavar='<audit>',
            dest='audit',
            help=_('UUID of the audit'))
        parser.add_argument(
            '--detail',
            dest='detail',
            action='store_true',
            default=False,
            help=_("Show detailed information about data model."))
        return parser

    def get_tuple(self, dic, fields):
        ret_tup = []
        for item in fields:
            ret_tup.append(dic.get(item))
        return tuple(ret_tup)

    def take_action(self, parsed_args):
        client = getattr(self.app.client_manager, "infra-optim")
        allowed_type = ['compute', 'storage', 'baremetal']
        params = {}
        if parsed_args.audit:
            params["audit"] = parsed_args.audit
        if parsed_args.type:
            if parsed_args.type not in allowed_type:
                raise exceptions.CommandError(
                    'Type %s error, '
                    'Please check the valid type!' % parsed_args.type)
            params["data_model_type"] = parsed_args.type
        try:
            data_model = client.data_model.list(**params)
        except exceptions.HTTPNotFound as exc:
            raise exceptions.CommandError(str(exc))
        # TODO(chenker) Add Storage MODEL_FIELDS when using Storage Datamodel.
        if parsed_args.detail:
            fields = res_fields.COMPUTE_MODEL_LIST_FIELDS
            field_labels = res_fields.COMPUTE_MODEL_LIST_FIELD_LABELS
        else:
            fields = res_fields.COMPUTE_MODEL_SHORT_LIST_FIELDS
            field_labels = res_fields.COMPUTE_MODEL_SHORT_LIST_FIELD_LABELS
        return (field_labels,
                (self.get_tuple(item, fields) for item in data_model.context))



#
# Copyright 2015 b<>com
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


# Audit Template
AUDIT_TEMPLATE_FIELDS = [
    'uuid', 'created_at', 'updated_at', 'deleted_at',
    'description', 'name', 'goal_name', 'strategy_name', 'scope']

AUDIT_TEMPLATE_FIELD_LABELS = [
    'UUID', 'Created At', 'Updated At', 'Deleted At',
    'Description', 'Name', 'Goal', 'Strategy', 'Audit Scope']

AUDIT_TEMPLATE_SHORT_LIST_FIELDS = [
    'uuid', 'name', 'goal_name', 'strategy_name']

AUDIT_TEMPLATE_SHORT_LIST_FIELD_LABELS = ['UUID', 'Name', 'Goal', 'Strategy']

# Audit
AUDIT_FIELDS = ['uuid', 'name', 'created_at', 'updated_at', 'deleted_at',
                'state', 'audit_type', 'parameters', 'interval', 'goal_name',
                'strategy_name', 'scope', 'auto_trigger', 'next_run_time',
                'hostname', 'start_time', 'end_time', 'force']

AUDIT_FIELD_LABELS = ['UUID', 'Name', 'Created At', 'Updated At', 'Deleted At',
                      'State', 'Audit Type', 'Parameters', 'Interval', 'Goal',
                      'Strategy', 'Audit Scope', 'Auto Trigger',
                      'Next Run Time', 'Hostname', 'Start Time', 'End Time',
                      'Force']

AUDIT_SHORT_LIST_FIELDS = ['uuid', 'name', 'audit_type',
                           'state', 'goal_name', 'strategy_name',
                           'auto_trigger']

AUDIT_SHORT_LIST_FIELD_LABELS = ['UUID', 'Name', 'Audit Type', 'State', 'Goal',
                                 'Strategy', 'Auto Trigger']

# Action Plan
ACTION_PLAN_FIELDS = ['uuid', 'created_at', 'updated_at', 'deleted_at',
                      'audit_uuid', 'strategy_name', 'state',
                      'efficacy_indicators', 'global_efficacy', 'hostname']

ACTION_PLAN_FIELD_LABELS = ['UUID', 'Created At', 'Updated At', 'Deleted At',
                            'Audit', 'Strategy', 'State',
                            'Efficacy indicators', 'Global efficacy',
                            'Hostname']

ACTION_PLAN_SHORT_LIST_FIELDS = ['uuid', 'audit_uuid', 'state',
                                 'updated_at', 'global_efficacy']

ACTION_PLAN_SHORT_LIST_FIELD_LABELS = ['UUID', 'Audit', 'State',
                                       'Updated At', 'Global efficacy']

GLOBAL_EFFICACY_FIELDS = ['value', 'unit', 'name', 'description']

# Action
ACTION_FIELDS = ['uuid', 'created_at', 'updated_at', 'deleted_at', 'parents',
                 'state', 'action_plan_uuid', 'action_type',
                 'input_parameters', 'description']

ACTION_FIELD_LABELS = ['UUID', 'Created At', 'Updated At', 'Deleted At',
                       'Parents', 'State', 'Action Plan', 'Action',
                       'Parameters', 'Description']

ACTION_SHORT_LIST_FIELDS = ['uuid', 'parents',
                            'state', 'action_plan_uuid', 'action_type']

ACTION_SHORT_LIST_FIELD_LABELS = ['UUID', 'Parents', 'State',
                                  'Action Plan', 'Action']
# Goals

GOAL_FIELDS = ['uuid', 'name', 'display_name', 'efficacy_specification']

GOAL_FIELD_LABELS = ['UUID', 'Name', 'Display name', 'Efficacy specification']

GOAL_SHORT_LIST_FIELDS = ['uuid', 'name', 'display_name']

GOAL_SHORT_LIST_FIELD_LABELS = ['UUID', 'Name', 'Display name']

# Strategies

STRATEGY_FIELDS = ['uuid', 'name', 'display_name', 'goal_name',
                   'parameters_spec']

STRATEGY_FIELD_LABELS = ['UUID', 'Name', 'Display name', 'Goal',
                         'Parameters spec']

# Data Model

COMPUTE_MODEL_LIST_FIELDS = [
    'server_uuid', 'server_name', 'server_vcpus',
    'server_memory', 'server_disk', 'server_state', 'node_uuid',
    'node_hostname', 'node_vcpus', 'node_vcpu_ratio', 'node_memory',
    'node_memory_ratio', 'node_disk', 'node_disk_ratio', 'node_state']

COMPUTE_MODEL_LIST_FIELD_LABELS = [
    'Server_UUID', 'Server Name', 'Server Vcpus',
    'Server Memory', 'Server Disk', 'Server State', 'Node UUID',
    'Node Host Name', 'Node Vcpus', 'Node Vcpu Ratio', 'Node Memory',
    'Node Memory Ratio', 'Node Disk', 'Node Disk Ratio', 'Node State']

COMPUTE_MODEL_SHORT_LIST_FIELDS = [
    'server_uuid', 'server_name',
    'server_state', 'node_uuid', 'node_hostname']

COMPUTE_MODEL_SHORT_LIST_FIELD_LABELS = [
    'Server UUID', 'Server Name',
    'Server State', 'Node UUID', 'Node Host Name']

STRATEGY_SHORT_LIST_FIELDS = ['uuid', 'name', 'display_name', 'goal_name']

STRATEGY_SHORT_LIST_FIELD_LABELS = ['UUID', 'Name', 'Display name', 'Goal']

STRATEGY_STATE_FIELDS = ['type', 'state', 'mandatory', 'comment']

STRATEGY_STATE_FIELD_LABELS = ['Type', 'State', 'Mandatory', 'Comment']

# Metric Collector
METRIC_COLLECTOR_FIELDS = ['uuid', 'created_at', 'updated_at', 'deleted_at',
                           'endpoint', 'category']

METRIC_COLLECTOR_FIELD_LABELS = ['UUID', 'Created At', 'Updated At',
                                 'Deleted At', 'Endpoint URL',
                                 'Metric Category']

METRIC_COLLECTOR_SHORT_LIST_FIELDS = ['uuid', 'endpoint', 'category']

METRIC_COLLECTOR_SHORT_LIST_FIELD_LABELS = ['UUID', 'Endpoint URL',
                                            'Metric Category']
# Scoring Engines
SCORING_ENGINE_FIELDS = ['uuid', 'name', 'description', 'metainfo']
SCORING_ENGINE_FIELD_LABELS = ['UUID', 'Name', 'Description', 'Metainfo']

SCORING_ENGINE_SHORT_LIST_FIELDS = ['uuid', 'name', 'description']
SCORING_ENGINE_SHORT_LIST_FIELD_LABELS = ['UUID', 'Name', 'Description']

# Services
SERVICE_FIELDS = ['id', 'name', 'host', 'status', 'last_seen_up']
SERVICE_FIELD_LABELS = ['ID', 'Name', 'Host', 'Status', 'Last seen up']
SERVICE_SHORT_LIST_FIELDS = ['id', 'name', 'host', 'status']
SERVICE_SHORT_LIST_FIELD_LABELS = ['ID', 'Name', 'Host', 'Status']



#
# Copyright 2013 IBM Corp
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

import datetime
from unittest import mock

import six

from watcherclient import exceptions
from watcherclient import shell
from watcherclient.tests.unit.v1 import base
from watcherclient import v1 as resource
from watcherclient.v1 import resource_fields

ACTION_1 = {
    'uuid': '770ef053-ecb3-48b0-85b5-d55a2dbc6588',
    'action_plan_uuid': 'f8e47706-efcf-49a4-a5c4-af604eb492f2',
    'state': 'PENDING',
    'action_type': 'migrate',
    'parents': ['239f02a5-9649-4e14-9d33-ac2bf67cb755'],
    'input_parameters': {"test": 1},
    'description': 'test',
    'created_at': datetime.datetime.now().isoformat(),
    'updated_at': None,
    'deleted_at': None,
}

ACTION_2 = {
    'uuid': '239f02a5-9649-4e14-9d33-ac2bf67cb755',
    'action_plan_uuid': 'f8e47706-efcf-49a4-a5c4-af604eb492f2',
    'state': 'PENDING',
    'action_type': 'migrate',
    'parents': ['67653274-eb24-c7ba-70f6-a84e73d80843'],
    'input_parameters': {"test": 2},
    'description': 'test',
    'created_at': datetime.datetime.now().isoformat(),
    'updated_at': None,
    'deleted_at': None,
}

ACTION_3 = {
    'uuid': '67653274-eb24-c7ba-70f6-a84e73d80843',
    'action_plan_uuid': 'a5199d0e-0702-4613-9234-5ae2af8dafea',
    'parents': [],
    'state': 'PENDING',
    'action_type': 'sleep',
    'description': 'test',
    'created_at': datetime.datetime.now().isoformat(),
    'updated_at': None,
    'deleted_at': None,
}

ACTION_PLAN_1 = {
    'uuid': 'a5199d0e-0702-4613-9234-5ae2af8dafea',
    'action': '770ef053-ecb3-48b0-85b5-d55a2dbc6588',
    'state': 'RECOMMENDED',
    'created_at': datetime.datetime.now().isoformat(),
    'updated_at': None,
    'deleted_at': None,
}


class ActionShellTest(base.CommandTestCase):

    SHORT_LIST_FIELDS = resource_fields.ACTION_SHORT_LIST_FIELDS
    SHORT_LIST_FIELD_LABELS = resource_fields.ACTION_SHORT_LIST_FIELD_LABELS
    FIELDS = resource_fields.ACTION_FIELDS
    FIELD_LABELS = resource_fields.ACTION_FIELD_LABELS

    def setUp(self):
        super(self.__class__, self).setUp()

        p_action_manager = mock.patch.object(resource, 'ActionManager')
        p_action_plan_manager = mock.patch.object(
            resource, 'ActionPlanManager')
        self.m_action_mgr_cls = p_action_manager.start()
        self.m_action_plan_mgr_cls = p_action_plan_manager.start()
        self.addCleanup(p_action_manager.stop)
        self.addCleanup(p_action_plan_manager.stop)

        self.m_action_mgr = mock.Mock()
        self.m_action_plan_mgr = mock.Mock()

        self.m_action_mgr_cls.return_value = self.m_action_mgr
        self.m_action_plan_mgr_cls.return_value = self.m_action_plan_mgr

        self.stdout = six.StringIO()
        self.cmd = shell.WatcherShell(stdout=self.stdout)

    def test_do_action_list(self):
        action1 = resource.Action(mock.Mock(), ACTION_1)
        action2 = resource.Action(mock.Mock(), ACTION_2)
        self.m_action_mgr.list.return_value = [action1, action2]

        exit_code, results = self.run_cmd('action list')

        self.assertEqual(0, exit_code)
        self.assertEqual(
            [self.resource_as_dict(action1, self.SHORT_LIST_FIELDS,
                                   self.SHORT_LIST_FIELD_LABELS),
             self.resource_as_dict(action2, self.SHORT_LIST_FIELDS,
                                   self.SHORT_LIST_FIELD_LABELS)],
            results)

        self.m_action_mgr.list.assert_called_once_with(detail=False)

    def test_do_action_list_detail(self):
        action1 = resource.Action(mock.Mock(), ACTION_1)
        action2 = resource.Action(mock.Mock(), ACTION_2)
        self.m_action_mgr.list.return_value = [action1, action2]

        exit_code, results = self.run_cmd('action list --detail')

        self.assertEqual(0, exit_code)
        self.assertEqual(
            [self.resource_as_dict(action1, self.FIELDS,
                                   self.FIELD_LABELS),
             self.resource_as_dict(action2, self.FIELDS,
                                   self.FIELD_LABELS)],
            results)

        self.m_action_mgr.list.assert_called_once_with(detail=True)

    def test_do_action_list_marker(self):
        action2 = resource.Action(mock.Mock(), ACTION_2)
        action3 = resource.Action(mock.Mock(), ACTION_3)
        self.m_action_mgr.list.return_value = [
            action2, action3]

        exit_code, results = self.run_cmd(
            'action list --marker 770ef053-ecb3-48b0-85b5-d55a2dbc6588')

        self.assertEqual(0, exit_code)
        self.assertEqual(
            [self.resource_as_dict(action2, self.SHORT_LIST_FIELDS,
                                   self.SHORT_LIST_FIELD_LABELS),
             self.resource_as_dict(action3, self.SHORT_LIST_FIELDS,
                                   self.SHORT_LIST_FIELD_LABELS)],
            results)

        self.m_action_mgr.list.assert_called_once_with(
            detail=False,
            marker='770ef053-ecb3-48b0-85b5-d55a2dbc6588')

    def test_do_action_show_by_uuid(self):
        action = resource.Action(mock.Mock(), ACTION_1)
        self.m_action_mgr.get.return_value = action
        self.m_action_plan_mgr.get.return_value = action

        exit_code, result = self.run_cmd(
            'action show 5869da81-4876-4687-a1ed-12cd64cf53d9')

        self.assertEqual(0, exit_code)
        self.assertEqual(
            self.resource_as_dict(action, self.FIELDS, self.FIELD_LABELS),
            result)
        self.m_action_mgr.get.assert_called_once_with(
            '5869da81-4876-4687-a1ed-12cd64cf53d9')

    def test_do_action_show_by_not_uuid(self):
        self.m_action_mgr.get.side_effect = exceptions.HTTPNotFound

        exit_code, result = self.run_cmd(
            'action show not_uuid', formatting=None)

        self.assertEqual(1, exit_code)
        self.assertEqual('', result)



# Copyright 2013 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools
from testtools import matchers

from watcherclient.tests.unit import utils
import watcherclient.v1.action

ACTION1 = {
    'id': 1,
    'uuid': '770ef053-ecb3-48b0-85b5-d55a2dbc6588',
    'action_plan': 'f8e47706-efcf-49a4-a5c4-af604eb492f2',
    'description': 'Action_1 description',
    'next': '239f02a5-9649-4e14-9d33-ac2bf67cb755',
    'state': 'PENDING',
}

ACTION2 = {
    'id': 2,
    'uuid': '239f02a5-9649-4e14-9d33-ac2bf67cb755',
    'action_plan': 'f8e47706-efcf-49a4-a5c4-af604eb492f2',
    'description': 'Action_2 description',
    'next': '67653274-eb24-c7ba-70f6-a84e73d80843',
    'state': 'PENDING',
}

ACTION3 = {
    'id': 3,
    'uuid': '67653274-eb24-c7ba-70f6-a84e73d80843',
    'action_plan': 'a5199d0e-0702-4613-9234-5ae2af8dafea',
    'description': 'Action_3 description',
    'next': None,
    'state': 'PENDING',
}

ACTION_PLAN1 = {
    'id': 1,
    'uuid': 'a5199d0e-0702-4613-9234-5ae2af8dafea',
    'audit': '770ef053-ecb3-48b0-85b5-d55a2dbc6588',
    'state': 'RECOMMENDED'
}

fake_responses = {
    '/v1/actions':
    {
        'GET': (
            {},
            {"actions": [ACTION1, ACTION2, ACTION3]},
        ),
    },
    '/v1/actions/?action_plan_uuid=%s' % ACTION1['action_plan']:
    {
        'GET': (
            {},
            {"actions": [ACTION1, ACTION2]},
        ),
    },
    '/v1/actions/?audit_uuid=%s' % ACTION_PLAN1['audit']:
    {
        'GET': (
            {},
            {"actions": [ACTION3]},
        ),
    },
    '/v1/actions/detail':
    {
        'GET': (
            {},
            {"actions": [ACTION1, ACTION2, ACTION3]},
        ),
    },
    '/v1/actions/%s' % ACTION1['uuid']:
    {
        'GET': (
            {},
            ACTION1,
        ),
        'DELETE': (
            {},
            None,
        ),
    },
    '/v1/actions/detail?action_plan_uuid=%s' % ACTION1['action_plan']:
    {
        'GET': (
            {},
            {"actions": [ACTION1, ACTION2]},
        ),
    },
    '/v1/actions/detail?audit_uuid=%s' % ACTION_PLAN1['audit']:
    {
        'GET': (
            {},
            {"actions": [ACTION3]},
        ),
    }
}

fake_responses_pagination = {
    '/v1/actions':
    {
        'GET': (
            {},
            {"actions": [ACTION1],
             "next": "http://127.0.0.1:9322/v1/actions/?limit=1"}
        ),
    },
    '/v1/actions/?limit=1':
    {
        'GET': (
            {},
            {"actions": [ACTION2]}
        ),
    },
}

fake_responses_sorting = {
    '/v1/actions/?sort_key=updated_at':
    {
        'GET': (
            {},
            {"actions": [ACTION3, ACTION2, ACTION1]}
        ),
    },
    '/v1/actions/?sort_dir=desc':
    {
        'GET': (
            {},
            {"actions": [ACTION3, ACTION2, ACTION1]}
        ),
    },
}

fake_responses_marker = {
    '/v1/actions/?marker=770ef053-ecb3-48b0-85b5-d55a2dbc6588':
    {
        'GET': (
            {},
            {"actions": [ACTION2, ACTION3]}
        ),
    },
}


class ActionManagerTest(testtools.TestCase):

    def setUp(self):
        super(ActionManagerTest, self).setUp()
        self.api = utils.FakeAPI(fake_responses)
        self.mgr = watcherclient.v1.action.ActionManager(self.api)

    def test_actions_list(self):
        actions = self.mgr.list()
        expect = [
            ('GET', '/v1/actions', {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(3, len(actions))

    def test_actions_list_by_action_plan(self):
        actions = self.mgr.list(action_plan=ACTION1['action_plan'])
        expect = [
            ('GET',
             '/v1/actions/?action_plan_uuid=%s' % ACTION1['action_plan'],
             {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(2, len(actions))

    def test_actions_list_detail(self):
        actions = self.mgr.list(detail=True)
        expect = [
            ('GET', '/v1/actions/detail', {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(3, len(actions))

    def test_actions_list_by_action_plan_detail(self):
        actions = self.mgr.list(action_plan=ACTION1['action_plan'],
                                detail=True)
        expect = [
            ('GET',
             '/v1/actions/detail?action_plan_uuid=%s' % ACTION1['action_plan'],
             {},
             None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(2, len(actions))

    def test_actions_list_limit(self):
        self.api = utils.FakeAPI(fake_responses_pagination)
        self.mgr = watcherclient.v1.action.ActionManager(self.api)
        actions = self.mgr.list(limit=1)
        expect = [
            ('GET', '/v1/actions/?limit=1', {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertThat(actions, matchers.HasLength(1))

    def test_actions_list_pagination_no_limit(self):
        self.api = utils.FakeAPI(fake_responses_pagination)
        self.mgr = watcherclient.v1.action.ActionManager(self.api)
        actions = self.mgr.list(limit=0)
        expect = [
            ('GET', '/v1/actions', {}, None),
            ('GET', '/v1/actions/?limit=1', {}, None)
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertThat(actions, matchers.HasLength(2))

    def test_actions_list_sort_key(self):
        self.api = utils.FakeAPI(fake_responses_sorting)
        self.mgr = watcherclient.v1.action.ActionManager(self.api)
        actions = self.mgr.list(sort_key='updated_at')
        expect = [
            ('GET', '/v1/actions/?sort_key=updated_at', {}, None)
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(3, len(actions))

    def test_actions_list_sort_dir(self):
        self.api = utils.FakeAPI(fake_responses_sorting)
        self.mgr = watcherclient.v1.action.ActionManager(self.api)
        actions = self.mgr.list(sort_dir='desc')
        expect = [
            ('GET', '/v1/actions/?sort_dir=desc', {}, None)
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(3, len(actions))

    def test_actions_list_marker(self):
        self.api = utils.FakeAPI(fake_responses_marker)
        self.mgr = watcherclient.v1.action.ActionManager(self.api)
        actions = self.mgr.list(
            marker='770ef053-ecb3-48b0-85b5-d55a2dbc6588')
        expect = [
            ('GET',
             '/v1/actions/?marker=770ef053-ecb3-48b0-85b5-d55a2dbc6588',
             {},
             None)
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(2, len(actions))

    def test_actions_show(self):
        action = self.mgr.get(ACTION1['uuid'])
        expect = [
            ('GET', '/v1/actions/%s' % ACTION1['uuid'], {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(ACTION1['uuid'], action.uuid)
        self.assertEqual(ACTION1['action_plan'], action.action_plan)
        self.assertEqual(ACTION1['next'], action.next)



#
# Copyright 2013 IBM Corp
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

import datetime
from unittest import mock

import six

from watcherclient import shell
from watcherclient.tests.unit.v1 import base
from watcherclient import v1 as resource
from watcherclient.v1 import resource_fields

GOAL_1 = {
    'uuid': "fc087747-61be-4aad-8126-b701731ae836",
    'name': "SERVER_CONSOLIDATION",
    'display_name': 'Server Consolidation',
    'created_at': datetime.datetime.now().isoformat(),
    'updated_at': None,
    'deleted_at': None,
}

STRATEGY_1 = {
    'uuid': '2cf86250-d309-4b81-818e-1537f3dba6e5',
    'name': 'basic',
    'display_name': 'Basic consolidation',
    'goal_uuid': 'fc087747-61be-4aad-8126-b701731ae836',
    'created_at': datetime.datetime.now().isoformat(),
    'updated_at': None,
    'deleted_at': None,
}

AUDIT_TEMPLATE_1 = {
    'uuid': 'f8e47706-efcf-49a4-a5c4-af604eb492f2',
    'name': 'at1',
    'description': 'Audit Template 1 description',
    'goal_uuid': 'fc087747-61be-4aad-8126-b701731ae836',
    'goal_name': 'SERVER_CONSOLIDATION',
    'strategy_uuid': '2cf86250-d309-4b81-818e-1537f3dba6e5',
    'strategy_name': 'basic',
    'created_at': datetime.datetime.now().isoformat(),
    'updated_at': None,
    'deleted_at': None,
    'scope': []
}

AUDIT_TEMPLATE_2 = {
    'uuid': '2a60ca9b-09b0-40ff-8674-de8a36fc4bc8',
    'name': 'at2',
    'description': 'Audit Template 2',
    'goal_uuid': 'fc087747-61be-4aad-8126-b701731ae836',
    'goal_name': 'SERVER_CONSOLIDATION',
    'strategy_uuid': None,
    'strategy_name': None,
    'created_at': datetime.datetime.now().isoformat(),
    'updated_at': None,
    'deleted_at': None,
    'scope': []
}


class AuditTemplateShellTest(base.CommandTestCase):

    SHORT_LIST_FIELDS = resource_fields.AUDIT_TEMPLATE_SHORT_LIST_FIELDS
    SHORT_LIST_FIELD_LABELS = (
        resource_fields.AUDIT_TEMPLATE_SHORT_LIST_FIELD_LABELS)
    FIELDS = resource_fields.AUDIT_TEMPLATE_FIELDS
    FIELD_LABELS = resource_fields.AUDIT_TEMPLATE_FIELD_LABELS

    def setUp(self):
        super(self.__class__, self).setUp()

        # goal mock
        p_goal_manager = mock.patch.object(resource, 'GoalManager')
        self.m_goal_mgr_cls = p_goal_manager.start()
        self.addCleanup(p_goal_manager.stop)

        self.m_goal_mgr = mock.Mock()
        self.m_goal_mgr_cls.return_value = self.m_goal_mgr

        # strategy mock
        p_strategy_manager = mock.patch.object(resource, 'StrategyManager')
        self.m_strategy_mgr_cls = p_strategy_manager.start()
        self.addCleanup(p_strategy_manager.stop)

        self.m_strategy_mgr = mock.Mock()
        self.m_strategy_mgr_cls.return_value = self.m_strategy_mgr

        # audit template mock
        p_audit_template_manager = mock.patch.object(
            resource, 'AuditTemplateManager')
        self.m_audit_template_mgr_cls = p_audit_template_manager.start()
        self.addCleanup(p_audit_template_manager.stop)

        self.m_audit_template_mgr = mock.Mock()
        self.m_audit_template_mgr_cls.return_value = self.m_audit_template_mgr

        # stdout mock
        self.stdout = six.StringIO()
        self.cmd = shell.WatcherShell(stdout=self.stdout)

    def test_do_audit_template_list(self):
        audit_template1 = resource.AuditTemplate(mock.Mock(), AUDIT_TEMPLATE_1)
        audit_template2 = resource.AuditTemplate(mock.Mock(), AUDIT_TEMPLATE_2)
        self.m_audit_template_mgr.list.return_value = [
            audit_template1, audit_template2]

        exit_code, results = self.run_cmd('audittemplate list')

        self.assertEqual(0, exit_code)
        self.assertEqual(
            [self.resource_as_dict(audit_template1, self.SHORT_LIST_FIELDS,
                                   self.SHORT_LIST_FIELD_LABELS),
             self.resource_as_dict(audit_template2, self.SHORT_LIST_FIELDS,
                                   self.SHORT_LIST_FIELD_LABELS)],
            results)

        self.m_audit_template_mgr.list.assert_called_once_with(detail=False)

    def test_do_audit_template_list_marker(self):
        audit_template2 = resource.AuditTemplate(mock.Mock(), AUDIT_TEMPLATE_2)
        self.m_audit_template_mgr.list.return_value = [audit_template2]

        exit_code, results = self.run_cmd(
            'audittemplate list --marker '
            'f8e47706-efcf-49a4-a5c4-af604eb492f2')

        self.assertEqual(0, exit_code)
        self.assertEqual(
            [self.resource_as_dict(audit_template2, self.SHORT_LIST_FIELDS,
                                   self.SHORT_LIST_FIELD_LABELS)],
            results)

        self.m_audit_template_mgr.list.assert_called_once_with(
            detail=False,
            marker='f8e47706-efcf-49a4-a5c4-af604eb492f2')

    def test_do_audit_template_list_detail(self):
        audit_template1 = resource.AuditTemplate(mock.Mock(), AUDIT_TEMPLATE_1)
        audit_template2 = resource.AuditTemplate(mock.Mock(), AUDIT_TEMPLATE_2)
        self.m_audit_template_mgr.list.return_value = [
            audit_template1, audit_template2]

        exit_code, results = self.run_cmd('audittemplate list --detail')

        self.assertEqual(0, exit_code)
        self.assertEqual(
            [self.resource_as_dict(audit_template1, self.FIELDS,
                                   self.FIELD_LABELS),
             self.resource_as_dict(audit_template2, self.FIELDS,
                                   self.FIELD_LABELS)],
            results)

        self.m_audit_template_mgr.list.assert_called_once_with(detail=True)

    def test_do_audit_template_list_filter_by_goal_uuid(self):
        audit_template1 = resource.AuditTemplate(mock.Mock(), AUDIT_TEMPLATE_1)
        audit_template2 = resource.AuditTemplate(mock.Mock(), AUDIT_TEMPLATE_2)
        self.m_audit_template_mgr.list.return_value = [
            audit_template1, audit_template2]

        exit_code, results = self.run_cmd(
            'audittemplate list --goal '
            'fc087747-61be-4aad-8126-b701731ae836')

        self.assertEqual(0, exit_code)
        self.assertEqual(
            [self.resource_as_dict(audit_template1, self.SHORT_LIST_FIELDS,
                                   self.SHORT_LIST_FIELD_LABELS),
             self.resource_as_dict(audit_template2, self.SHORT_LIST_FIELDS,
                                   self.SHORT_LIST_FIELD_LABELS)],
            results)

        self.m_audit_template_mgr.list.assert_called_once_with(
            detail=False,
            goal='fc087747-61be-4aad-8126-b701731ae836',
        )

    def test_do_audit_template_list_filter_by_goal_name(self):
        goal1 = resource.Goal(mock.Mock(), GOAL_1)
        strategy1 = resource.Strategy(mock.Mock(), STRATEGY_1)
        audit_template1 = resource.AuditTemplate(mock.Mock(), AUDIT_TEMPLATE_1)
        audit_template2 = resource.AuditTemplate(mock.Mock(), AUDIT_TEMPLATE_2)
        self.m_goal_mgr.get.return_value = goal1
        self.m_strategy_mgr.get.return_value = strategy1
        self.m_audit_template_mgr.list.return_value = [
            audit_template1, audit_template2]

        exit_code, results = self.run_cmd(
            'audittemplate list --goal SERVER_CONSOLIDATION')

        self.assertEqual(0, exit_code)
        self.assertEqual(
            [self.resource_as_dict(audit_template1, self.SHORT_LIST_FIELDS,
                                   self.SHORT_LIST_FIELD_LABELS),
             self.resource_as_dict(audit_template2, self.SHORT_LIST_FIELDS,
                                   self.SHORT_LIST_FIELD_LABELS)],
            results)

        self.m_audit_template_mgr.list.assert_called_once_with(
            detail=False,
            goal='SERVER_CONSOLIDATION',
        )

    def test_do_audit_template_list_filter_by_strategy_uuid(self):
        goal1 = resource.Goal(mock.Mock(), GOAL_1)
        strategy1 = resource.Strategy(mock.Mock(), STRATEGY_1)
        audit_template1 = resource.AuditTemplate(mock.Mock(), AUDIT_TEMPLATE_1)
        self.m_goal_mgr.get.return_value = goal1
        self.m_strategy_mgr.get.return_value = strategy1
        self.m_audit_template_mgr.list.return_value = [audit_template1]

        exit_code, results = self.run_cmd(
            'audittemplate list --strategy '
            '2cf86250-d309-4b81-818e-1537f3dba6e5')

        self.assertEqual(0, exit_code)
        self.assertEqual(
            [self.resource_as_dict(audit_template1, self.SHORT_LIST_FIELDS,
                                   self.SHORT_LIST_FIELD_LABELS)],
            results)

        self.m_audit_template_mgr.list.assert_called_once_with(
            detail=False,
            strategy='2cf86250-d309-4b81-818e-1537f3dba6e5',
        )

    def test_do_audit_template_list_filter_by_strategy_name(self):
        audit_template1 = resource.AuditTemplate(mock.Mock(), AUDIT_TEMPLATE_1)
        self.m_audit_template_mgr.list.return_value = [audit_template1]

        exit_code, results = self.run_cmd(
            'audittemplate list --strategy '
            'basic')

        self.assertEqual(0, exit_code)
        self.assertEqual(
            [self.resource_as_dict(audit_template1, self.SHORT_LIST_FIELDS,
                                   self.SHORT_LIST_FIELD_LABELS)],
            results)

        self.m_audit_template_mgr.list.assert_called_once_with(
            detail=False,
            strategy='basic',
        )

    def test_do_audit_template_show_by_name(self):
        audit_template = resource.AuditTemplate(mock.Mock(), AUDIT_TEMPLATE_1)
        self.m_audit_template_mgr.get.return_value = audit_template

        exit_code, result = self.run_cmd('audittemplate show at1')

        self.assertEqual(0, exit_code)
        self.assertEqual(self.resource_as_dict(audit_template, self.FIELDS,
                                               self.FIELD_LABELS),
                         result)
        self.m_audit_template_mgr.get.assert_called_once_with('at1')

    def test_do_audit_template_show_by_uuid(self):
        audit_template = resource.AuditTemplate(mock.Mock(), AUDIT_TEMPLATE_1)
        self.m_audit_template_mgr.get.return_value = audit_template

        exit_code, result = self.run_cmd(
            'audittemplate show f8e47706-efcf-49a4-a5c4-af604eb492f2')

        self.assertEqual(0, exit_code)
        self.assertEqual(self.resource_as_dict(audit_template, self.FIELDS,
                                               self.FIELD_LABELS),
                         result)
        self.m_audit_template_mgr.get.assert_called_once_with(
            'f8e47706-efcf-49a4-a5c4-af604eb492f2')

    def test_do_audit_template_delete(self):
        self.m_audit_template_mgr.delete.return_value = ''

        exit_code, result = self.run_cmd(
            'audittemplate delete f8e47706-efcf-49a4-a5c4-af604eb492f2',
            formatting=None)

        self.assertEqual(0, exit_code)
        self.assertEqual('', result)
        self.m_audit_template_mgr.delete.assert_called_once_with(
            'f8e47706-efcf-49a4-a5c4-af604eb492f2')

    def test_do_audit_template_delete_multiple(self):
        self.m_audit_template_mgr.delete.return_value = ''

        exit_code, result = self.run_cmd(
            'audittemplate delete f8e47706-efcf-49a4-a5c4-af604eb492f2 '
            '92dfce2f-0a5e-473f-92b7-d92e21839e4d',
            formatting=None)

        self.assertEqual(0, exit_code)
        self.assertEqual('', result)
        self.m_audit_template_mgr.delete.assert_any_call(
            'f8e47706-efcf-49a4-a5c4-af604eb492f2')
        self.m_audit_template_mgr.delete.assert_any_call(
            '92dfce2f-0a5e-473f-92b7-d92e21839e4d')

    def test_do_audit_template_update(self):
        audit_template = resource.AuditTemplate(mock.Mock(), AUDIT_TEMPLATE_1)
        self.m_audit_template_mgr.update.return_value = audit_template

        exit_code, result = self.run_cmd(
            'audittemplate update at1 replace description="New description"')

        self.assertEqual(0, exit_code)
        self.assertEqual(self.resource_as_dict(audit_template, self.FIELDS,
                                               self.FIELD_LABELS),
                         result)
        self.m_audit_template_mgr.update.assert_called_once_with(
            'at1',
            [{'op': 'replace', 'path': '/description',
              'value': 'New description'}])

    def test_do_audit_template_create(self):
        audit_template = resource.AuditTemplate(mock.Mock(), AUDIT_TEMPLATE_1)
        self.m_audit_template_mgr.create.return_value = audit_template

        exit_code, result = self.run_cmd(
            'audittemplate create at1 fc087747-61be-4aad-8126-b701731ae836')

        self.assertEqual(0, exit_code)
        self.assertEqual(self.resource_as_dict(audit_template, self.FIELDS,
                                               self.FIELD_LABELS),
                         result)
        self.m_audit_template_mgr.create.assert_called_once_with(
            goal='fc087747-61be-4aad-8126-b701731ae836',
            name='at1')

    def test_do_audit_template_create_with_description(self):
        audit_template = resource.AuditTemplate(mock.Mock(), AUDIT_TEMPLATE_1)
        self.m_audit_template_mgr.create.return_value = audit_template

        exit_code, result = self.run_cmd(
            'audittemplate create at1 fc087747-61be-4aad-8126-b701731ae836 '
            '-d "Audit Template 1 description"')

        self.assertEqual(0, exit_code)
        self.assertEqual(self.resource_as_dict(audit_template, self.FIELDS,
                                               self.FIELD_LABELS),
                         result)
        self.m_audit_template_mgr.create.assert_called_once_with(
            goal='fc087747-61be-4aad-8126-b701731ae836',
            name='at1',
            description='Audit Template 1 description')

    def test_do_audit_template_create_with_aggregate(self):
        audit_template = resource.AuditTemplate(mock.Mock(), AUDIT_TEMPLATE_1)
        self.m_audit_template_mgr.create.return_value = audit_template

        exit_code, result = self.run_cmd(
            'audittemplate create at1 fc087747-61be-4aad-8126-b701731ae836')

        self.assertEqual(0, exit_code)
        self.assertEqual(self.resource_as_dict(audit_template, self.FIELDS,
                                               self.FIELD_LABELS),
                         result)
        self.m_audit_template_mgr.create.assert_called_once_with(
            goal='fc087747-61be-4aad-8126-b701731ae836',
            name='at1')



# Copyright 2013 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

import testtools
from testtools import matchers

from watcherclient.tests.unit import utils
import watcherclient.v1.audit


AUDIT1 = {
    'id': 1,
    'uuid': '5869da81-4876-4687-a1ed-12cd64cf53d9',
    'audit_type': 'ONE_SHOT',
    'goal': 'fc087747-61be-4aad-8126-b701731ae836',
    'strategy': '2cf86250-d309-4b81-818e-1537f3dba6e5',
}

AUDIT2 = {
    'id': 2,
    'uuid': 'a5199d0e-0702-4613-9234-5ae2af8dafea',
    'audit_type': 'ONE_SHOT',
    'goal': 'fc087747-61be-4aad-8126-b701731ae836',
    'strategy': None,
}


CREATE_AUDIT = copy.deepcopy(AUDIT1)
del CREATE_AUDIT['id']
del CREATE_AUDIT['uuid']

UPDATED_AUDIT1 = copy.deepcopy(AUDIT1)
NEW_STATE = 'SUCCESS'
UPDATED_AUDIT1['state'] = NEW_STATE

fake_responses = {
    '/v1/audits':
    {
        'GET': (
            {},
            {"audits": [AUDIT1]},
        ),
        'POST': (
            {},
            CREATE_AUDIT,
        ),
    },
    '/v1/audits/detail':
    {
        'GET': (
            {},
            {"audits": [AUDIT1]},
        ),
    },
    '/v1/audits/%s' % AUDIT1['uuid']:
    {
        'GET': (
            {},
            AUDIT1,
        ),
        'DELETE': (
            {},
            None,
        ),
        'PATCH': (
            {},
            UPDATED_AUDIT1,
        ),
    },
    '/v1/audits/detail?uuid=%s' % AUDIT1['uuid']:
    {
        'GET': (
            {},
            {"audits": [AUDIT1]},
        ),
    },
}

fake_responses_pagination = {
    '/v1/audits':
    {
        'GET': (
            {},
            {"audits": [AUDIT1],
             "next": "http://127.0.0.1:9322/v1/audits/?limit=1"}
        ),
    },
    '/v1/audits/?limit=1':
    {
        'GET': (
            {},
            {"audits": [AUDIT2]}
        ),
    },
}

fake_responses_sorting = {
    '/v1/audits/?sort_key=updated_at':
    {
        'GET': (
            {},
            {"audits": [AUDIT2, AUDIT1]}
        ),
    },
    '/v1/audits/?sort_dir=desc':
    {
        'GET': (
            {},
            {"audits": [AUDIT2, AUDIT1]}
        ),
    },
}

fake_responses_goal = {
    '/v1/audits/?goal=dummy':
    {
        'GET': (
            {},
            {"audits": [AUDIT2, AUDIT1]}
        ),
    },
}

fake_responses_strategy = {
    '/v1/audits/?strategy=dummy':
    {
        'GET': (
            {},
            {"audits": [AUDIT1]}
        ),
    },
}

fake_responses_marker = {
    '/v1/audits/?marker=5869da81-4876-4687-a1ed-12cd64cf53d9':
    {
        'GET': (
            {},
            {"audits": [AUDIT2]}
        ),
    },
}


class AuditManagerTest(testtools.TestCase):
    def setUp(self):
        super(AuditManagerTest, self).setUp()
        self.api = utils.FakeAPI(fake_responses)
        self.mgr = watcherclient.v1.audit.AuditManager(self.api)

    def test_audits_list(self):
        audits = self.mgr.list()
        expect = [
            ('GET', '/v1/audits', {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(1, len(audits))

    def test_audits_list_detail(self):
        audits = self.mgr.list(detail=True)
        expect = [
            ('GET', '/v1/audits/detail', {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(1, len(audits))

    def test_audits_list_limit(self):
        self.api = utils.FakeAPI(fake_responses_pagination)
        self.mgr = watcherclient.v1.audit.AuditManager(self.api)
        audits = self.mgr.list(limit=1)
        expect = [
            ('GET', '/v1/audits/?limit=1', {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertThat(audits, matchers.HasLength(1))

    def test_audits_list_pagination_no_limit(self):
        self.api = utils.FakeAPI(fake_responses_pagination)
        self.mgr = watcherclient.v1.audit.AuditManager(self.api)
        audits = self.mgr.list(limit=0)
        expect = [
            ('GET', '/v1/audits', {}, None),
            ('GET', '/v1/audits/?limit=1', {}, None)
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertThat(audits, matchers.HasLength(2))

    def test_audits_list_sort_key(self):
        self.api = utils.FakeAPI(fake_responses_sorting)
        self.mgr = watcherclient.v1.audit.AuditManager(self.api)
        audits = self.mgr.list(sort_key='updated_at')
        expect = [
            ('GET', '/v1/audits/?sort_key=updated_at', {}, None)
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(2, len(audits))

    def test_audits_list_sort_dir(self):
        self.api = utils.FakeAPI(fake_responses_sorting)
        self.mgr = watcherclient.v1.audit.AuditManager(self.api)
        audits = self.mgr.list(sort_dir='desc')
        expect = [
            ('GET', '/v1/audits/?sort_dir=desc', {}, None)
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(2, len(audits))

    def test_audits_list_goal(self):
        self.api = utils.FakeAPI(fake_responses_goal)
        self.mgr = watcherclient.v1.audit.AuditManager(self.api)
        audits = self.mgr.list(goal='dummy')
        expect = [
            ('GET', '/v1/audits/?goal=dummy', {}, None)
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(2, len(audits))

    def test_audits_list_strategy(self):
        self.api = utils.FakeAPI(fake_responses_strategy)
        self.mgr = watcherclient.v1.audit.AuditManager(self.api)
        audits = self.mgr.list(strategy='dummy')
        expect = [
            ('GET', '/v1/audits/?strategy=dummy', {}, None)
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(1, len(audits))

    def test_audits_list_marker(self):
        self.api = utils.FakeAPI(fake_responses_marker)
        self.mgr = watcherclient.v1.audit.AuditManager(self.api)
        audits = self.mgr.list(marker=AUDIT1['uuid'])
        expect = [
            ('GET', '/v1/audits/?marker=5869da81-4876-4687-a1ed-12cd64cf53d9',
             {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(1, len(audits))

    def test_audits_show(self):
        audit = self.mgr.get(AUDIT1['uuid'])
        expect = [
            ('GET', '/v1/audits/%s' % AUDIT1['uuid'], {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(AUDIT1['uuid'], audit.uuid)

    def test_create(self):
        audit = self.mgr.create(**CREATE_AUDIT)
        expect = [
            ('POST', '/v1/audits', {}, CREATE_AUDIT),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertTrue(audit)



# Copyright (c) 2016 Servionica
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from osc_lib import utils

from watcherclient._i18n import _
from watcherclient.common import command
from watcherclient.common import utils as common_utils
from watcherclient import exceptions
from watcherclient.v1 import resource_fields as res_fields


class ShowService(command.ShowOne):
    """Show detailed information about a given service."""

    def get_parser(self, prog_name):
        parser = super(ShowService, self).get_parser(prog_name)
        parser.add_argument(
            'service',
            metavar='<service>',
            help=_('ID or name of the service'),
        )
        return parser

    def take_action(self, parsed_args):
        client = getattr(self.app.client_manager, "infra-optim")

        try:
            service = client.service.get(parsed_args.service)
        except exceptions.HTTPNotFound as exc:
            raise exceptions.CommandError(str(exc))

        columns = res_fields.SERVICE_FIELDS
        column_headers = res_fields.SERVICE_FIELD_LABELS

        return column_headers, utils.get_item_properties(service, columns)


class ListService(command.Lister):
    """List information on retrieved services."""

    def get_parser(self, prog_name):
        parser = super(ListService, self).get_parser(prog_name)
        parser.add_argument(
            '--detail',
            dest='detail',
            action='store_true',
            default=False,
            help=_("Show detailed information about each service."))
        parser.add_argument(
            '--limit',
            metavar='<limit>',
            type=int,
            help=_('Maximum number of services to return per request, '
                   '0 for no limit. Default is the maximum number used '
                   'by the Watcher API Service.'))
        parser.add_argument(
            '--sort-key',
            metavar='<field>',
            help=_('Goal field that will be used for sorting.'))
        parser.add_argument(
            '--sort-dir',
            metavar='<direction>',
            choices=['asc', 'desc'],
            help='Sort direction: "asc" (the default) or "desc".')

        return parser

    def take_action(self, parsed_args):
        client = getattr(self.app.client_manager, "infra-optim")

        params = {}
        if parsed_args.detail:
            fields = res_fields.SERVICE_FIELDS
            field_labels = res_fields.SERVICE_FIELD_LABELS
        else:
            fields = res_fields.SERVICE_SHORT_LIST_FIELDS
            field_labels = res_fields.SERVICE_SHORT_LIST_FIELD_LABELS

        params.update(
            common_utils.common_params_for_list(
                parsed_args, fields, field_labels))

        try:
            data = client.service.list(**params)
        except exceptions.HTTPNotFound as ex:
            raise exceptions.CommandError(str(ex))

        return (field_labels,
                (utils.get_item_properties(item, fields) for item in data))



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.


import logging

from osc_lib import utils

import watcherclient
from watcherclient.common import api_versioning
from watcherclient.common import httpclient
from watcherclient import exceptions

LOG = logging.getLogger(__name__)

DEFAULT_API_VERSION = httpclient.LATEST_VERSION
API_VERSION_OPTION = 'os_infra_optim_api_version'
API_NAME = 'infra-optim'
API_VERSIONS = {
    '1': 'watcherclient.v1.client.Client',
}


def make_client(instance):
    """Returns an infra-optim service client."""

    version = api_versioning.APIVersion(instance._api_version[API_NAME])

    infraoptim_client_class = utils.get_client_class(
        API_NAME,
        version.ver_major,
        API_VERSIONS)
    LOG.debug('Instantiating infraoptim client: %s', infraoptim_client_class)

    client = infraoptim_client_class(
        os_infra_optim_api_version=instance._api_version[API_NAME],
        session=instance.session,
        region_name=instance._region_name,
    )

    return client


def build_option_parser(parser):
    """Hook to add global options."""
    parser.add_argument('--os-infra-optim-api-version',
                        metavar='<infra-optim-api-version>',
                        default=utils.env(
                            'OS_INFRA_OPTIM_API_VERSION',
                            default=DEFAULT_API_VERSION),
                        help=('Watcher API version, default=' +
                              DEFAULT_API_VERSION +
                              ' (Env: OS_INFRA_OPTIM_API_VERSION)'))
    return parser


def check_api_version(check_version):
    """Validate version supplied by user

    Returns:
    * True if version is OK
    * False if the version has not been checked and the previous plugin
    check should be performed
    * throws an exception if the version is no good
    """

    infra_api_version = api_versioning.get_api_version(check_version)

    # Bypass X.latest format microversion
    if not infra_api_version.is_latest():
        if infra_api_version > api_versioning.APIVersion("2.0"):
            if not infra_api_version.matches(
                watcherclient.API_MIN_VERSION,
                watcherclient.API_MAX_VERSION,
            ):
                msg = "versions supported by client: %(min)s - %(max)s" % {
                    "min": watcherclient.API_MIN_VERSION.get_string(),
                    "max": watcherclient.API_MAX_VERSION.get_string(),
                }
                raise exceptions.CommandError(msg)
    return True



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_serialization import jsonutils
from oslo_utils import uuidutils

from keystoneauth1.fixture import v2 as ks_v2_fixture
from keystoneauth1.fixture import v3 as ks_v3_fixture

# these are copied from python-keystoneclient tests
BASE_HOST = 'http://keystone.example.com'
BASE_URL = "%s:5000/" % BASE_HOST
UPDATED = '2013-03-06T00:00:00Z'

V2_URL = "%sv2.0" % BASE_URL
V2_DESCRIBED_BY_HTML = {'href': 'http://docs.openstack.org/api/'
                                'openstack-identity-service/2.0/content/',
                        'rel': 'describedby',
                        'type': 'text/html'}
V2_DESCRIBED_BY_PDF = {'href': 'http://docs.openstack.org/api/openstack-ident'
                               'ity-service/2.0/identity-dev-guide-2.0.pdf',
                       'rel': 'describedby',
                       'type': 'application/pdf'}

V2_VERSION = {'id': 'v2.0',
              'links': [{'href': V2_URL, 'rel': 'self'},
                        V2_DESCRIBED_BY_HTML, V2_DESCRIBED_BY_PDF],
              'status': 'stable',
              'updated': UPDATED}

V3_URL = "%sv3" % BASE_URL
V3_MEDIA_TYPES = [{'base': 'application/json',
                   'type': 'application/vnd.openstack.identity-v3+json'},
                  {'base': 'application/xml',
                   'type': 'application/vnd.openstack.identity-v3+xml'}]

V3_VERSION = {'id': 'v3.0',
              'links': [{'href': V3_URL, 'rel': 'self'}],
              'media-types': V3_MEDIA_TYPES,
              'status': 'stable',
              'updated': UPDATED}

TOKENID = uuidutils.generate_uuid(dashed=False)


def _create_version_list(versions):
    return jsonutils.dumps({'versions': {'values': versions}})


def _create_single_version(version):
    return jsonutils.dumps({'version': version})


V3_VERSION_LIST = _create_version_list([V3_VERSION, V2_VERSION])
V2_VERSION_LIST = _create_version_list([V2_VERSION])

V3_VERSION_ENTRY = _create_single_version(V3_VERSION)
V2_VERSION_ENTRY = _create_single_version(V2_VERSION)


def keystone_request_callback(request, uri, headers):
    response_headers = {"content-type": "application/json"}
    token_id = TOKENID
    if uri == BASE_URL:
        return (200, headers, V3_VERSION_LIST)
    elif uri == BASE_URL + "/v2.0":
        v2_token = ks_v2_fixture.Token(token_id)
        return (200, response_headers, jsonutils.dumps(v2_token))
    elif uri == BASE_URL + "/v3":
        v3_token = ks_v3_fixture.Token()
        response_headers["X-Subject-Token"] = token_id
        return (201, response_headers, jsonutils.dumps(v3_token))



#
# Copyright 2013 Red Hat, Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from osc_lib import utils
import six

from watcherclient._i18n import _
from watcherclient.common import command
from watcherclient.common import utils as common_utils
from watcherclient import exceptions
from watcherclient.v1 import resource_fields as res_fields


class ShowGoal(command.ShowOne):
    """Show detailed information about a given goal."""

    def get_parser(self, prog_name):
        parser = super(ShowGoal, self).get_parser(prog_name)
        parser.add_argument(
            'goal',
            metavar='<goal>',
            help=_('UUID or name of the goal'),
        )
        return parser

    def _format_indicator_spec_table(self, spec, parsed_args):
        out = six.StringIO()
        self.formatter.emit_one(
            column_names=list(field.capitalize() for field in spec.keys()),
            data=utils.get_dict_properties(spec, spec.keys()),
            stdout=out,
            parsed_args=parsed_args,
            )
        return out.getvalue() or ''

    def take_action(self, parsed_args):
        client = getattr(self.app.client_manager, "infra-optim")

        try:
            goal = client.goal.get(parsed_args.goal)
        except exceptions.HTTPNotFound as exc:
            raise exceptions.CommandError(str(exc))

        columns = res_fields.GOAL_FIELDS
        column_headers = res_fields.GOAL_FIELD_LABELS

        if parsed_args.formatter == 'table':
            indicator_specs = ''
            # Format complex data types:
            for indicator_spec in goal.efficacy_specification:
                indicator_specs += self._format_indicator_spec_table(
                    indicator_spec, parsed_args)
            # Update the raw efficacy specs with the formatted one
            goal.efficacy_specification = indicator_specs

        return column_headers, utils.get_item_properties(goal, columns)


class ListGoal(command.Lister):
    """List information on retrieved goals."""

    def get_parser(self, prog_name):
        parser = super(ListGoal, self).get_parser(prog_name)
        parser.add_argument(
            '--detail',
            dest='detail',
            action='store_true',
            default=False,
            help=_("Show detailed information about each goal."))
        parser.add_argument(
            '--limit',
            metavar='<limit>',
            type=int,
            help=_('Maximum number of goals to return per request, '
                   '0 for no limit. Default is the maximum number used '
                   'by the Watcher API Service.'))
        parser.add_argument(
            '--sort-key',
            metavar='<field>',
            help=_('Goal field that will be used for sorting.'))
        parser.add_argument(
            '--sort-dir',
            metavar='<direction>',
            choices=['asc', 'desc'],
            help=_('Sort direction: "asc" (the default) or "desc".'))
        parser.add_argument(
            '--marker',
            dest='marker',
            metavar='<marker>',
            default=None,
            help=_('UUID of the last goal in the previous page; '
                   'displays list of goals after "marker".'))

        return parser

    def _format_indicator_spec_table(self, goal, parsed_args):
        out = six.StringIO()
        efficacy_specification = goal.efficacy_specification
        fields = ['name', 'unit']
        self.formatter.emit_list(
            column_names=list(field.capitalize()
                              for field in fields),
            data=[utils.get_dict_properties(spec, fields)
                  for spec in efficacy_specification],
            stdout=out,
            parsed_args=parsed_args,
            )
        return out.getvalue() or ''

    def take_action(self, parsed_args):
        client = getattr(self.app.client_manager, "infra-optim")

        if parsed_args.detail:
            fields = res_fields.GOAL_FIELDS
            field_labels = res_fields.GOAL_FIELD_LABELS
        else:
            fields = res_fields.GOAL_SHORT_LIST_FIELDS
            field_labels = res_fields.GOAL_SHORT_LIST_FIELD_LABELS

        params = {}
        params.update(
            common_utils.common_params_for_list(
                parsed_args, fields, field_labels))

        try:
            data = client.goal.list(**params)
        except exceptions.HTTPNotFound as ex:
            raise exceptions.CommandError(str(ex))

        if parsed_args.formatter == 'table':
            for goal in data:
                # Update the raw efficacy specs with the formatted one
                goal.efficacy_specification = (
                    self._format_indicator_spec_table(goal, parsed_args))

        return (field_labels,
                (utils.get_item_properties(item, fields) for item in data))



#
# Copyright 2013 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from watcherclient.common.apiclient import exceptions as exc
from watcherclient.common import utils
from watcherclient.tests.unit import utils as test_utils


class UtilsTest(test_utils.BaseTestCase):

    def test_args_array_to_dict(self):
        my_args = {
            'matching_metadata': ['str=foo', 'int=1', 'bool=true',
                                  'list=[1, 2, 3]', 'dict={"foo": "bar"}'],
            'other': 'value'
        }
        cleaned_dict = utils.args_array_to_dict(my_args,
                                                "matching_metadata")
        self.assertEqual({
            'matching_metadata': {'str': 'foo', 'int': 1, 'bool': True,
                                  'list': [1, 2, 3], 'dict': {'foo': 'bar'}},
            'other': 'value'
        }, cleaned_dict)

    def test_args_array_to_patch(self):
        my_args = {
            'attributes': ['str=foo', 'int=1', 'bool=true',
                           'list=[1, 2, 3]', 'dict={"foo": "bar"}'],
            'op': 'add',
        }
        patch = utils.args_array_to_patch(my_args['op'],
                                          my_args['attributes'])
        self.assertEqual([{'op': 'add', 'value': 'foo', 'path': '/str'},
                          {'op': 'add', 'value': 1, 'path': '/int'},
                          {'op': 'add', 'value': True, 'path': '/bool'},
                          {'op': 'add', 'value': [1, 2, 3], 'path': '/list'},
                          {'op': 'add', 'value': {"foo": "bar"},
                           'path': '/dict'}], patch)

    def test_args_array_to_patch_format_error(self):
        my_args = {
            'attributes': ['foobar'],
            'op': 'add',
        }
        self.assertRaises(exc.CommandError, utils.args_array_to_patch,
                          my_args['op'], my_args['attributes'])

    def test_args_array_to_patch_remove(self):
        my_args = {
            'attributes': ['/foo', 'extra/bar'],
            'op': 'remove',
        }
        patch = utils.args_array_to_patch(my_args['op'],
                                          my_args['attributes'])
        self.assertEqual([{'op': 'remove', 'path': '/foo'},
                          {'op': 'remove', 'path': '/extra/bar'}], patch)

    def test_split_and_deserialize(self):
        ret = utils.split_and_deserialize('str=foo')
        self.assertEqual(('str', 'foo'), ret)

        ret = utils.split_and_deserialize('int=1')
        self.assertEqual(('int', 1), ret)

        ret = utils.split_and_deserialize('bool=false')
        self.assertEqual(('bool', False), ret)

        ret = utils.split_and_deserialize('list=[1, "foo", 2]')
        self.assertEqual(('list', [1, "foo", 2]), ret)

        ret = utils.split_and_deserialize('dict={"foo": 1}')
        self.assertEqual(('dict', {"foo": 1}), ret)

        ret = utils.split_and_deserialize('str_int="1"')
        self.assertEqual(('str_int', "1"), ret)

    def test_split_and_deserialize_fail(self):
        self.assertRaises(exc.CommandError,
                          utils.split_and_deserialize, 'foo:bar')


class CommonParamsForListTest(test_utils.BaseTestCase):
    def setUp(self):
        super(CommonParamsForListTest, self).setUp()
        self.args = mock.Mock(limit=None, marker=None,
                              sort_key=None, sort_dir=None)
        self.args.detail = False
        self.expected_params = {'detail': False}

    def test_nothing_set(self):
        self.assertEqual(self.expected_params,
                         utils.common_params_for_list(self.args, [], []))

    def test_limit(self):
        self.args.limit = 42
        self.expected_params.update({'limit': 42})
        self.assertEqual(self.expected_params,
                         utils.common_params_for_list(self.args, [], []))

    def test_invalid_limit(self):
        self.args.limit = -42
        self.assertRaises(exc.CommandError,
                          utils.common_params_for_list,
                          self.args, [], [])

    def test_marker(self):
        self.args.marker = 'e420a881-d7df-4de2-bbf3-378cc13d9b3a'
        self.expected_params.update(
            {'marker': 'e420a881-d7df-4de2-bbf3-378cc13d9b3a'})
        self.assertEqual(self.expected_params,
                         utils.common_params_for_list(self.args, [], []))

    def test_sort_key_and_sort_dir(self):
        self.args.sort_key = 'field'
        self.args.sort_dir = 'desc'
        self.expected_params.update({'sort_key': 'field', 'sort_dir': 'desc'})
        self.assertEqual(self.expected_params,
                         utils.common_params_for_list(self.args,
                                                      ['field'],
                                                      []))

    def test_sort_key_allows_label(self):
        self.args.sort_key = 'Label'
        self.expected_params.update({'sort_key': 'field'})
        self.assertEqual(self.expected_params,
                         utils.common_params_for_list(self.args,
                                                      ['field', 'field2'],
                                                      ['Label', 'Label2']))

    def test_sort_key_invalid(self):
        self.args.sort_key = 'something'
        self.assertRaises(exc.CommandError,
                          utils.common_params_for_list,
                          self.args,
                          ['field', 'field2'],
                          [])

    def test_sort_dir_invalid(self):
        self.args.sort_dir = 'something'
        self.assertRaises(exc.CommandError,
                          utils.common_params_for_list,
                          self.args,
                          [],
                          [])

    def test_detail(self):
        self.args.detail = True
        self.expected_params['detail'] = True
        self.assertEqual(self.expected_params,
                         utils.common_params_for_list(self.args, [], []))


class CommonFiltersTest(test_utils.BaseTestCase):
    def test_limit(self):
        result = utils.common_filters(limit=42)
        self.assertEqual(['limit=42'], result)

    def test_limit_0(self):
        result = utils.common_filters(limit=0)
        self.assertEqual([], result)

    def test_other(self):
        for key in ('sort_key', 'sort_dir'):
            result = utils.common_filters(**{key: 'test'})
            self.assertEqual(['%s=test' % key], result)



# Copyright (c) 2016 Intel
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import datetime
from unittest import mock

import six

from watcherclient import shell
from watcherclient.tests.unit.v1 import base
from watcherclient import v1 as resource
from watcherclient.v1 import resource_fields

SCORING_ENGINE_1 = {
    'uuid': '5b558998-57ed-11e6-9ca8-08002722cb22',
    'name': 'se-01',
    'description': 'Scoring Engine 0.1',
    'metainfo': '{ "columns": ["cpu", "mem", "pci"] }',
    'created_at': datetime.datetime.now().isoformat(),
    'updated_at': None,
    'deleted_at': None,
}

SCORING_ENGINE_2 = {
    'uuid': '1f856554-57ee-11e6-ac72-08002722cb22',
    'name': 'se-02',
    'description': 'Some other Scoring Engine',
    'metainfo': 'mode=simplified',
    'created_at': datetime.datetime.now().isoformat(),
    'updated_at': None,
    'deleted_at': None,
}


class ScoringEngineShellTest(base.CommandTestCase):

    SHORT_LIST_FIELDS = resource_fields.SCORING_ENGINE_SHORT_LIST_FIELDS
    SHORT_LIST_FIELD_LABELS = (
        resource_fields.SCORING_ENGINE_SHORT_LIST_FIELD_LABELS)
    FIELDS = resource_fields.SCORING_ENGINE_FIELDS
    FIELD_LABELS = resource_fields.SCORING_ENGINE_FIELD_LABELS

    def setUp(self):
        super(self.__class__, self).setUp()

        p_se_manager = mock.patch.object(
            resource, 'ScoringEngineManager')
        self.m_se_mgr_cls = p_se_manager.start()
        self.addCleanup(p_se_manager.stop)

        self.m_se_mgr = mock.Mock()
        self.m_se_mgr_cls.return_value = self.m_se_mgr

        self.stdout = six.StringIO()
        self.cmd = shell.WatcherShell(stdout=self.stdout)

    def test_do_scoringengine_list(self):
        se1 = resource.ScoringEngine(mock.Mock(), SCORING_ENGINE_1)
        se2 = resource.ScoringEngine(mock.Mock(), SCORING_ENGINE_2)
        self.m_se_mgr.list.return_value = [
            se1, se2]

        exit_code, results = self.run_cmd('scoringengine list')

        self.assertEqual(0, exit_code)
        self.assertEqual(
            [self.resource_as_dict(se1, self.SHORT_LIST_FIELDS,
                                   self.SHORT_LIST_FIELD_LABELS),
             self.resource_as_dict(se2, self.SHORT_LIST_FIELDS,
                                   self.SHORT_LIST_FIELD_LABELS)],
            results)

        self.m_se_mgr.list.assert_called_once_with(detail=False)

    def test_do_scoringengine_list_detail(self):
        se1 = resource.Goal(mock.Mock(), SCORING_ENGINE_1)
        se2 = resource.Goal(mock.Mock(), SCORING_ENGINE_2)
        self.m_se_mgr.list.return_value = [
            se1, se2]

        exit_code, results = self.run_cmd('scoringengine list --detail')

        self.assertEqual(0, exit_code)
        self.assertEqual(
            [self.resource_as_dict(se1, self.FIELDS, self.FIELD_LABELS),
             self.resource_as_dict(se2, self.FIELDS, self.FIELD_LABELS)],
            results)

        self.m_se_mgr.list.assert_called_once_with(detail=True)

    def test_do_scoringengine_show_by_name(self):
        scoringengine = resource.Goal(mock.Mock(), SCORING_ENGINE_1)
        self.m_se_mgr.get.return_value = scoringengine

        exit_code, result = self.run_cmd('scoringengine show se-01')

        self.assertEqual(0, exit_code)
        self.assertEqual(
            self.resource_as_dict(scoringengine, self.FIELDS,
                                  self.FIELD_LABELS),
            result)
        self.m_se_mgr.get.assert_called_once_with('se-01')

    def test_do_scoringengine_show_by_uuid(self):
        scoringengine = resource.Goal(mock.Mock(), SCORING_ENGINE_1)
        self.m_se_mgr.get.return_value = scoringengine

        exit_code, result = self.run_cmd(
            'scoringengine show 5b558998-57ed-11e6-9ca8-08002722cb22')

        self.assertEqual(0, exit_code)
        self.assertEqual(
            self.resource_as_dict(scoringengine, self.FIELDS,
                                  self.FIELD_LABELS),
            result)
        self.m_se_mgr.get.assert_called_once_with(
            '5b558998-57ed-11e6-9ca8-08002722cb22')



# Copyright (c) 2016 b<>com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import datetime
from unittest import mock

from oslo_serialization import jsonutils
import six

from watcherclient import shell
from watcherclient.tests.unit.v1 import base
from watcherclient import v1 as resource
from watcherclient.v1 import resource_fields

STRATEGY_1 = {
    'uuid': '2cf86250-d309-4b81-818e-1537f3dba6e5',
    'name': 'basic',
    'display_name': 'Basic consolidation',
    'goal_uuid': 'fc087747-61be-4aad-8126-b701731ae836',
    'goal_name': 'SERVER_CONSOLIDATION',
    'created_at': datetime.datetime.now().isoformat(),
    'updated_at': None,
    'deleted_at': None,
    'parameters_spec': {},
}

STRATEGY_2 = {
    'uuid': 'b20bb987-ea8f-457a-a4ea-ab3ffdfeff8b',
    'name': 'dummy',
    'display_name': 'Dummy',
    'goal_uuid': '407b03b1-63c6-49b2-adaf-4df5c0090047',
    'goal_name': 'DUMMY',
    'created_at': datetime.datetime.now().isoformat(),
    'updated_at': None,
    'deleted_at': None,
    'parameters_spec': {},
}


class StrategyShellTest(base.CommandTestCase):

    SHORT_LIST_FIELDS = resource_fields.STRATEGY_SHORT_LIST_FIELDS
    SHORT_LIST_FIELD_LABELS = (
        resource_fields.STRATEGY_SHORT_LIST_FIELD_LABELS)
    FIELDS = resource_fields.STRATEGY_FIELDS
    FIELD_LABELS = resource_fields.STRATEGY_FIELD_LABELS
    STATE_FIELDS = resource_fields.STRATEGY_STATE_FIELDS
    STATE_FIELD_LABELS = resource_fields.STRATEGY_STATE_FIELD_LABELS

    def setUp(self):
        super(self.__class__, self).setUp()

        p_strategy_manager = mock.patch.object(resource, 'StrategyManager')
        self.m_strategy_mgr_cls = p_strategy_manager.start()
        self.addCleanup(p_strategy_manager.stop)

        self.m_strategy_mgr = mock.Mock()
        self.m_strategy_mgr_cls.return_value = self.m_strategy_mgr

        self.stdout = six.StringIO()
        self.cmd = shell.WatcherShell(stdout=self.stdout)

    def test_do_strategy_list(self):
        strategy1 = resource.Strategy(mock.Mock(), STRATEGY_1)
        strategy2 = resource.Strategy(mock.Mock(), STRATEGY_2)
        self.m_strategy_mgr.list.return_value = [
            strategy1, strategy2]

        exit_code, results = self.run_cmd('strategy list')

        self.assertEqual(0, exit_code)
        self.assertEqual(
            [self.resource_as_dict(strategy1, self.SHORT_LIST_FIELDS,
                                   self.SHORT_LIST_FIELD_LABELS),
             self.resource_as_dict(strategy2, self.SHORT_LIST_FIELDS,
                                   self.SHORT_LIST_FIELD_LABELS)],
            results)

        self.m_strategy_mgr.list.assert_called_once_with(detail=False)

    def test_do_strategy_list_marker(self):
        strategy2 = resource.Strategy(mock.Mock(), STRATEGY_2)
        self.m_strategy_mgr.list.return_value = [strategy2]

        exit_code, results = self.run_cmd(
            'strategy list --marker 2cf86250-d309-4b81-818e-1537f3dba6e5')

        self.assertEqual(0, exit_code)
        self.assertEqual(
            [self.resource_as_dict(strategy2, self.SHORT_LIST_FIELDS,
                                   self.SHORT_LIST_FIELD_LABELS)],
            results)

        self.m_strategy_mgr.list.assert_called_once_with(
            detail=False,
            marker='2cf86250-d309-4b81-818e-1537f3dba6e5')

    def test_do_strategy_list_detail(self):
        strategy1 = resource.Strategy(mock.Mock(), STRATEGY_1)
        strategy2 = resource.Strategy(mock.Mock(), STRATEGY_2)
        self.m_strategy_mgr.list.return_value = [
            strategy1, strategy2]

        exit_code, results = self.run_cmd('strategy list --detail')

        self.assertEqual(0, exit_code)
        self.assertEqual(
            [self.resource_as_dict(strategy1, self.FIELDS,
                                   self.FIELD_LABELS),
             self.resource_as_dict(strategy2, self.FIELDS,
                                   self.FIELD_LABELS)],
            results)

        self.m_strategy_mgr.list.assert_called_once_with(detail=True)

    def test_do_strategy_list_filter_by_goal_name(self):
        strategy2 = resource.Strategy(mock.Mock(), STRATEGY_2)
        self.m_strategy_mgr.list.return_value = [strategy2]

        exit_code, results = self.run_cmd(
            'strategy list --goal '
            'DUMMY')

        self.assertEqual(0, exit_code)
        self.assertEqual(
            [self.resource_as_dict(strategy2, self.SHORT_LIST_FIELDS,
                                   self.SHORT_LIST_FIELD_LABELS)],
            results)

        self.m_strategy_mgr.list.assert_called_once_with(
            detail=False,
            goal='DUMMY',
        )

    def test_do_strategy_list_filter_by_goal_uuid(self):
        strategy1 = resource.Strategy(mock.Mock(), STRATEGY_1)
        self.m_strategy_mgr.list.return_value = [strategy1]

        exit_code, results = self.run_cmd(
            'strategy list --goal '
            'fc087747-61be-4aad-8126-b701731ae836')

        self.assertEqual(0, exit_code)
        self.assertEqual(
            [self.resource_as_dict(strategy1, self.SHORT_LIST_FIELDS,
                                   self.SHORT_LIST_FIELD_LABELS)],
            results)

        self.m_strategy_mgr.list.assert_called_once_with(
            detail=False,
            goal='fc087747-61be-4aad-8126-b701731ae836',
        )

    def test_do_strategy_show_by_uuid(self):
        strategy = resource.Strategy(mock.Mock(), STRATEGY_1)
        self.m_strategy_mgr.get.return_value = strategy

        exit_code, result = self.run_cmd(
            'strategy show f8e47706-efcf-49a4-a5c4-af604eb492f2')

        self.assertEqual(0, exit_code)
        self.assertEqual(
            self.resource_as_dict(strategy, self.FIELDS, self.FIELD_LABELS),
            result)
        self.m_strategy_mgr.get.assert_called_once_with(
            'f8e47706-efcf-49a4-a5c4-af604eb492f2')

    def test_do_strategy_state(self):
        strategy1 = resource.Strategy(mock.Mock(), STRATEGY_1)
        strategy_req = [
            {'type': 'Datasource', 'mandatory': True,
             'comment': '', 'state': 'gnocchi: True'},
            {'type': 'Metrics', 'mandatory': False,
             'comment': '', 'state': jsonutils.dumps([
                 {'compute.node.cpu.percent': 'available'},
                 {'cpu_util': 'available'},
                 {'memory.resident': 'available'},
                 {'hardware.memory.used': 'not available'}])},
            {'type': 'CDM', 'mandatory': True,
             'comment': '',
             'state': jsonutils.dumps([{'compute_model': 'available'},
                                      {'storage_model': 'not available'}])},
            {'type': 'Name', 'mandatory': '', 'comment': '',
             'state': strategy1.name}]
        requirements = [resource.Strategy(mock.Mock(), req)
                        for req in strategy_req]
        self.m_strategy_mgr.state.return_value = requirements

        exit_code, results = self.run_cmd('strategy state basic')

        self.assertEqual(0, exit_code)
        self.assertEqual(
            [self.resource_as_dict(req, self.STATE_FIELDS,
                                   self.STATE_FIELD_LABELS)
             for req in requirements],
            results)



#
# Copyright 2016 Intel
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


import testtools
from testtools import matchers

from watcherclient.tests.unit import utils
import watcherclient.v1.scoring_engine

SE1 = {
    'uuid': '5b558998-57ed-11e6-9ca8-08002722cb22',
    'name': 'se-01',
    'description': 'Some Scoring Engine'
}

SE2 = {
    'uuid': '1f856554-57ee-11e6-ac72-08002722cb22',
    'name': 'se-02',
    'description': 'Some Other Scoring Engine'
}

fake_responses = {
    '/v1/scoring_engines':
    {
        'GET': (
            {},
            {"scoring_engines": [SE1]},
        ),
    },
    '/v1/scoring_engines/detail':
    {
        'GET': (
            {},
            {"scoring_engines": [SE1]},
        )
    },
    '/v1/scoring_engines/%s' % SE1['uuid']:
    {
        'GET': (
            {},
            SE1,
        ),
    },
    '/v1/scoring_engines/%s' % SE1['name']:
    {
        'GET': (
            {},
            SE1,
        ),
    },
}

fake_responses_pagination = {
    '/v1/scoring_engines':
    {
        'GET': (
            {},
            {"scoring_engines": [SE1],
             "next": "http://127.0.0.1:9322/v1/scoring_engines/?limit=1"}
        ),
    },
    '/v1/scoring_engines/?limit=1':
    {
        'GET': (
            {},
            {"scoring_engines": [SE2]}
        ),
    },
}

fake_responses_sorting = {
    '/v1/scoring_engines/?sort_key=id':
    {
        'GET': (
            {},
            {"scoring_engines": [SE1, SE2]}
        ),
    },
    '/v1/scoring_engines/?sort_dir=desc':
    {
        'GET': (
            {},
            {"scoring_engines": [SE2, SE1]}
        ),
    },
}


class ScoringEngineManagerTest(testtools.TestCase):

    def setUp(self):
        super(ScoringEngineManagerTest, self).setUp()
        self.api = utils.FakeAPI(fake_responses)
        self.mgr = \
            watcherclient.v1.scoring_engine.ScoringEngineManager(self.api)

    def test_scoring_engines_list(self):
        scoring_engines = self.mgr.list()
        expect = [
            ('GET', '/v1/scoring_engines', {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(1, len(scoring_engines))

    def test_scoring_engines_list_detail(self):
        scoring_engines = self.mgr.list(detail=True)
        expect = [
            ('GET', '/v1/scoring_engines/detail', {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(1, len(scoring_engines))

    def test_scoring_engines_list_limit(self):
        self.api = utils.FakeAPI(fake_responses_pagination)
        self.mgr = \
            watcherclient.v1.scoring_engine.ScoringEngineManager(self.api)
        scoring_engines = self.mgr.list(limit=1)
        expect = [
            ('GET', '/v1/scoring_engines/?limit=1', {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertThat(scoring_engines, matchers.HasLength(1))

    def test_scoring_engines_list_pagination_no_limit(self):
        self.api = utils.FakeAPI(fake_responses_pagination)
        self.mgr = \
            watcherclient.v1.scoring_engine.ScoringEngineManager(self.api)
        scoring_engines = self.mgr.list(limit=0)
        expect = [
            ('GET', '/v1/scoring_engines', {}, None),
            ('GET', '/v1/scoring_engines/?limit=1', {}, None)
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertThat(scoring_engines, matchers.HasLength(2))

    def test_scoring_engines_list_sort_key(self):
        self.api = utils.FakeAPI(fake_responses_sorting)
        self.mgr = \
            watcherclient.v1.scoring_engine.ScoringEngineManager(self.api)
        scoring_engines = self.mgr.list(sort_key='id')
        expect = [
            ('GET', '/v1/scoring_engines/?sort_key=id', {}, None)
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(2, len(scoring_engines))

    def test_scoring_engines_list_sort_dir(self):
        self.api = utils.FakeAPI(fake_responses_sorting)
        self.mgr = \
            watcherclient.v1.scoring_engine.ScoringEngineManager(self.api)
        scoring_engines = self.mgr.list(sort_dir='desc')
        expect = [
            ('GET', '/v1/scoring_engines/?sort_dir=desc', {}, None)
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(2, len(scoring_engines))

    def test_scoring_engines_show(self):
        scoring_engine = self.mgr.get(SE1['uuid'])
        expect = [
            ('GET', '/v1/scoring_engines/%s' % SE1['uuid'], {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(SE1['uuid'], scoring_engine.uuid)

    def test_scoring_engines_show_by_name(self):
        scoring_engine = self.mgr.get(SE1['name'])
        expect = [
            ('GET', '/v1/scoring_engines/%s' % SE1['name'], {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(SE1['name'], scoring_engine.name)



# Copyright 2013 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


import testtools
from testtools import matchers

from watcherclient.tests.unit import utils
import watcherclient.v1.goal

GOAL1 = {
    'uuid': "fc087747-61be-4aad-8126-b701731ae836",
    'name': "SERVER_CONSOLIDATION",
    'display_name': 'Server Consolidation'
}

GOAL2 = {
    'uuid': "407b03b1-63c6-49b2-adaf-4df5c0090047",
    'name': "COST_OPTIMIZATION",
    'display_name': 'Cost Optimization'
}

fake_responses = {
    '/v1/goals':
    {
        'GET': (
            {},
            {"goals": [GOAL1]},
        ),
    },
    '/v1/goals/detail':
    {
        'GET': (
            {},
            {"goals": [GOAL1]},
        )
    },
    '/v1/goals/%s' % GOAL1['uuid']:
    {
        'GET': (
            {},
            GOAL1,
        ),
    },
    '/v1/goals/%s' % GOAL1['name']:
    {
        'GET': (
            {},
            GOAL1,
        ),
    },
}

fake_responses_pagination = {
    '/v1/goals':
    {
        'GET': (
            {},
            {"goals": [GOAL1],
             "next": "http://127.0.0.1:9322/v1/goals/?limit=1"}
        ),
    },
    '/v1/goals/?limit=1':
    {
        'GET': (
            {},
            {"goals": [GOAL2]}
        ),
    },
}

fake_responses_sorting = {
    '/v1/goals/?sort_key=id':
    {
        'GET': (
            {},
            {"goals": [GOAL1, GOAL2]}
        ),
    },
    '/v1/goals/?sort_dir=desc':
    {
        'GET': (
            {},
            {"goals": [GOAL2, GOAL1]}
        ),
    },
}

fake_responses_marker = {
    '/v1/goals/?marker=fc087747-61be-4aad-8126-b701731ae836':
    {
        'GET': (
            {},
            {"goals": [GOAL2]}
        ),
    },
}


class GoalManagerTest(testtools.TestCase):

    def setUp(self):
        super(GoalManagerTest, self).setUp()
        self.api = utils.FakeAPI(fake_responses)
        self.mgr = watcherclient.v1.goal.GoalManager(self.api)

    def test_goals_list(self):
        goals = self.mgr.list()
        expect = [
            ('GET', '/v1/goals', {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(1, len(goals))

    def test_goals_list_detail(self):
        goals = self.mgr.list(detail=True)
        expect = [
            ('GET', '/v1/goals/detail', {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(1, len(goals))

    def test_goals_list_limit(self):
        self.api = utils.FakeAPI(fake_responses_pagination)
        self.mgr = watcherclient.v1.goal.GoalManager(self.api)
        goals = self.mgr.list(limit=1)
        expect = [
            ('GET', '/v1/goals/?limit=1', {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertThat(goals, matchers.HasLength(1))

    def test_goals_list_marker(self):
        self.api = utils.FakeAPI(fake_responses_marker)
        self.mgr = watcherclient.v1.goal.GoalManager(self.api)
        goals = self.mgr.list(marker=GOAL1['uuid'])
        expect = [
            ('GET', '/v1/goals/?marker=fc087747-61be-4aad-8126-b701731ae836',
             {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(1, len(goals))

    def test_goals_list_pagination_no_limit(self):
        self.api = utils.FakeAPI(fake_responses_pagination)
        self.mgr = watcherclient.v1.goal.GoalManager(self.api)
        goals = self.mgr.list(limit=0)
        expect = [
            ('GET', '/v1/goals', {}, None),
            ('GET', '/v1/goals/?limit=1', {}, None)
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertThat(goals, matchers.HasLength(2))

    def test_goals_list_sort_key(self):
        self.api = utils.FakeAPI(fake_responses_sorting)
        self.mgr = watcherclient.v1.goal.GoalManager(self.api)
        goals = self.mgr.list(sort_key='id')
        expect = [
            ('GET', '/v1/goals/?sort_key=id', {}, None)
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(2, len(goals))

    def test_goals_list_sort_dir(self):
        self.api = utils.FakeAPI(fake_responses_sorting)
        self.mgr = watcherclient.v1.goal.GoalManager(self.api)
        goals = self.mgr.list(sort_dir='desc')
        expect = [
            ('GET', '/v1/goals/?sort_dir=desc', {}, None)
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(2, len(goals))

    def test_goals_show(self):
        goal = self.mgr.get(GOAL1['uuid'])
        expect = [
            ('GET', '/v1/goals/%s' % GOAL1['uuid'], {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(GOAL1['uuid'], goal.uuid)

    def test_goals_show_by_name(self):
        goal = self.mgr.get(GOAL1['name'])
        expect = [
            ('GET', '/v1/goals/%s' % GOAL1['name'], {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(GOAL1['name'], goal.name)



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging
import os
import pkgutil
import re

from oslo_utils import strutils

from watcherclient._i18n import _
from watcherclient import exceptions

LOG = logging.getLogger(__name__)
if not LOG.handlers:
    LOG.addHandler(logging.StreamHandler())


MINOR_1_START_END_TIMING = '1.1'
MINOR_2_FORCE_AUDIT = '1.2'
HEADER_NAME = "OpenStack-API-Version"
# key is a deprecated version and value is an alternative version.
DEPRECATED_VERSIONS = {}

_type_error_msg = _("'%(other)s' should be an instance of '%(cls)s'")


def allow_start_end_audit_time(requested_version):
    """Check if we should support optional start/end attributes for Audit.

    Version 1.1 of the API added support for start and end time of continuous
    audits.
    """
    return (APIVersion(requested_version) >=
            APIVersion(MINOR_1_START_END_TIMING))


def launch_audit_forced(requested_version):
    """Check if we should support force option for Audit.

    Version 1.2 of the API added support for force option.
    """
    return (APIVersion(requested_version) >=
            APIVersion(MINOR_2_FORCE_AUDIT))


class APIVersion(object):
    """This class represents an API Version Request.

    This class provides convenience methods for manipulation
    and comparison of version numbers that we need to do to
    implement microversions.
    """

    def __init__(self, version_str=None):
        """Create an API version object.

        :param version_str: String representation of APIVersionRequest.
                            Correct format is 'X.Y', where 'X' and 'Y'
                            are int values. None value should be used
                            to create Null APIVersionRequest, which is
                            equal to 0.0
        """
        self.ver_major = 0
        self.ver_minor = 0

        if version_str is not None:
            match = re.match(r"^([1-9]\d*)\.([1-9]\d*|0|latest)$", version_str)
            if match:
                self.ver_major = int(match.group(1))
                if match.group(2) == "latest":
                    # NOTE(andreykurilin): Infinity allows to easily determine
                    # latest version and doesn't require any additional checks
                    # in comparison methods.
                    self.ver_minor = float("inf")
                else:
                    self.ver_minor = int(match.group(2))
            else:
                msg = _("Invalid format of client version '%s'. "
                        "Expected format 'X.Y', where X is a major part and Y "
                        "is a minor part of version.") % version_str
                raise exceptions.UnsupportedVersion(msg)

    def __str__(self):
        """Debug/Logging representation of object."""
        if self.is_latest():
            return "Latest API Version Major: %s" % self.ver_major
        return ("API Version Major: %s, Minor: %s"
                % (self.ver_major, self.ver_minor))

    def __repr__(self):
        if self.is_null():
            return "<APIVersion: null>"
        else:
            return "<APIVersion: %s>" % self.get_string()

    def is_null(self):
        return self.ver_major == 0 and self.ver_minor == 0

    def is_latest(self):
        return self.ver_minor == float("inf")

    def __lt__(self, other):
        if not isinstance(other, APIVersion):
            raise TypeError(_type_error_msg % {"other": other,
                                               "cls": self.__class__})

        return ((self.ver_major, self.ver_minor) <
                (other.ver_major, other.ver_minor))

    def __eq__(self, other):
        if not isinstance(other, APIVersion):
            raise TypeError(_type_error_msg % {"other": other,
                                               "cls": self.__class__})

        return ((self.ver_major, self.ver_minor) ==
                (other.ver_major, other.ver_minor))

    def __gt__(self, other):
        if not isinstance(other, APIVersion):
            raise TypeError(_type_error_msg % {"other": other,
                                               "cls": self.__class__})

        return ((self.ver_major, self.ver_minor) >
                (other.ver_major, other.ver_minor))

    def __le__(self, other):
        return self < other or self == other

    def __ne__(self, other):
        return not self.__eq__(other)

    def __ge__(self, other):
        return self > other or self == other

    def matches(self, min_version, max_version):
        """Matches the version object.

        Returns whether the version object represents a version
        greater than or equal to the minimum version and less than
        or equal to the maximum version.

        :param min_version: Minimum acceptable version.
        :param max_version: Maximum acceptable version.
        :returns: boolean

        If min_version is null then there is no minimum limit.
        If max_version is null then there is no maximum limit.
        If self is null then raise ValueError
        """

        if self.is_null():
            raise ValueError(_("Null APIVersion doesn't support 'matches'."))
        if max_version.is_null() and min_version.is_null():
            return True
        elif max_version.is_null():
            return min_version <= self
        elif min_version.is_null():
            return self <= max_version
        else:
            return min_version <= self <= max_version

    def get_string(self):
        """Version string representation.

        Converts object to string representation which if used to create
        an APIVersion object results in the same version.
        """
        if self.is_null():
            raise ValueError(
                _("Null APIVersion cannot be converted to string."))
        elif self.is_latest():
            return "%s.%s" % (self.ver_major, "latest")
        return "%s.%s" % (self.ver_major, self.ver_minor)


def get_available_major_versions():
    # NOTE(andreykurilin): available clients version should not be
    # hardcoded, so let's discover them.
    matcher = re.compile(r"v[0-9]*$")
    submodules = pkgutil.iter_modules(
        [os.path.dirname(os.path.dirname(__file__))])
    available_versions = [name[1:] for loader, name, ispkg in submodules
                          if matcher.search(name)]
    return available_versions


def check_major_version(api_version):
    """Checks major part of ``APIVersion`` obj is supported.

    :raises watcherclient.exceptions.UnsupportedVersion: if major part is not
                                                      supported
    """
    available_versions = get_available_major_versions()
    if (not api_version.is_null() and
            str(api_version.ver_major) not in available_versions):
        if len(available_versions) == 1:
            msg = _("Invalid client version '%(version)s'. "
                    "Major part should be '%(major)s'") % {
                        "version": api_version.get_string(),
                        "major": available_versions[0]}
        else:
            msg = _("Invalid client version '%(version)s'. "
                    "Major part must be one of: '%(major)s'") % {
                        "version": api_version.get_string(),
                        "major": ", ".join(available_versions)}
        raise exceptions.UnsupportedVersion(msg)


def get_api_version(version_string):
    """Returns checked APIVersion object"""
    version_string = str(version_string)
    if version_string in DEPRECATED_VERSIONS:
        LOG.warning(
            "Version %(deprecated_version)s is deprecated, using "
            "alternative version %(alternative)s instead.",
            {"deprecated_version": version_string,
             "alternative": DEPRECATED_VERSIONS[version_string]})
        version_string = DEPRECATED_VERSIONS[version_string]
    if strutils.is_int_like(version_string):
        version_string = "%s.0" % version_string

    api_version = APIVersion(version_string)
    check_major_version(api_version)
    return api_version



# Copyright 2019 ZTE Corporation.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from watcherclient.common import base
from watcherclient.common import utils


class DataModel(base.Resource):
    def __repr__(self):
        return "<DataModel %s>" % self._info


class DataModelManager(base.Manager):
    resource_class = DataModel

    @staticmethod
    def _path(filters=None):
        if filters:
            path = '/v1/data_model/%s' % filters
        else:
            path = '/v1/data_model'
        return path

    def list(self, data_model_type='compute', audit=None):
        """Retrieve a list of data model.

        :param data_model_type: The type of data model user wants to list.
                                Supported values: compute.
                                Future support values: storage, baremetal.
                                The default value is compute.
        :param audit: The UUID of the audit, used to filter data model
                      by the scope in audit.

        :returns: A list of data model.

        """
        path = ''
        filters = utils.common_filters()

        if audit:
            filters.append('audit_uuid=%s' % audit)
        filters.append('data_model_type=%s' % data_model_type)

        path += '?' + '&'.join(filters)

        return self._list(self._path(path))[0]



# Copyright (c) 2016 b<>com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import datetime
from unittest import mock

import six

from watcherclient import shell
from watcherclient.tests.unit.v1 import base
from watcherclient import v1 as resource
from watcherclient.v1 import resource_fields

GOAL_1 = {
    'uuid': "fc087747-61be-4aad-8126-b701731ae836",
    'name': "SERVER_CONSOLIDATION",
    'display_name': 'Server Consolidation',
    'efficacy_specification': [
        {'description': 'Indicator 1', 'name': 'indicator1',
         'schema': 'Range(min=0, max=100, min_included=True, '
                   'max_included=True, msg=None)',
         'unit': '%'}
    ],
    'created_at': datetime.datetime.now().isoformat(),
    'updated_at': None,
    'deleted_at': None,
}

GOAL_2 = {
    'uuid': "407b03b1-63c6-49b2-adaf-4df5c0090047",
    'name': "COST_OPTIMIZATION",
    'display_name': 'Cost Optimization',
    'efficacy_specification': [
        {'description': 'Indicator 2', 'name': 'indicator2',
         'schema': 'Range(min=0, max=100, min_included=True, '
                   'max_included=True, msg=None)',
         'unit': '%'}
    ],
    'created_at': datetime.datetime.now().isoformat(),
    'updated_at': None,
    'deleted_at': None,
}


class GoalShellTest(base.CommandTestCase):

    SHORT_LIST_FIELDS = resource_fields.GOAL_SHORT_LIST_FIELDS
    SHORT_LIST_FIELD_LABELS = (
        resource_fields.GOAL_SHORT_LIST_FIELD_LABELS)
    FIELDS = resource_fields.GOAL_FIELDS
    FIELD_LABELS = resource_fields.GOAL_FIELD_LABELS

    def setUp(self):
        super(self.__class__, self).setUp()

        p_goal_manager = mock.patch.object(
            resource, 'GoalManager')
        self.m_goal_mgr_cls = p_goal_manager.start()
        self.addCleanup(p_goal_manager.stop)

        self.m_goal_mgr = mock.Mock()
        self.m_goal_mgr_cls.return_value = self.m_goal_mgr

        self.stdout = six.StringIO()
        self.cmd = shell.WatcherShell(stdout=self.stdout)

    def test_do_goal_list(self):
        goal1 = resource.Goal(mock.Mock(), GOAL_1)
        goal2 = resource.Goal(mock.Mock(), GOAL_2)
        self.m_goal_mgr.list.return_value = [
            goal1, goal2]

        exit_code, results = self.run_cmd('goal list')

        self.assertEqual(0, exit_code)
        self.assertEqual(
            [self.resource_as_dict(goal1, self.SHORT_LIST_FIELDS,
                                   self.SHORT_LIST_FIELD_LABELS),
             self.resource_as_dict(goal2, self.SHORT_LIST_FIELDS,
                                   self.SHORT_LIST_FIELD_LABELS)],
            results)

        self.m_goal_mgr.list.assert_called_once_with(detail=False)

    def test_do_goal_list_marker(self):
        goal2 = resource.Goal(mock.Mock(), GOAL_2)
        self.m_goal_mgr.list.return_value = [goal2]

        exit_code, results = self.run_cmd(
            'goal list --marker fc087747-61be-4aad-8126-b701731ae836')

        self.assertEqual(0, exit_code)
        self.assertEqual(
            [self.resource_as_dict(goal2, self.SHORT_LIST_FIELDS,
                                   self.SHORT_LIST_FIELD_LABELS)],
            results)

        self.m_goal_mgr.list.assert_called_once_with(
            detail=False,
            marker='fc087747-61be-4aad-8126-b701731ae836')

    def test_do_goal_list_detail(self):
        goal1 = resource.Goal(mock.Mock(), GOAL_1)
        goal2 = resource.Goal(mock.Mock(), GOAL_2)
        self.m_goal_mgr.list.return_value = [
            goal1, goal2]

        exit_code, results = self.run_cmd('goal list --detail')

        self.assertEqual(0, exit_code)
        self.assertEqual(
            [self.resource_as_dict(goal1, self.FIELDS, self.FIELD_LABELS),
             self.resource_as_dict(goal2, self.FIELDS, self.FIELD_LABELS)],
            results)

        self.m_goal_mgr.list.assert_called_once_with(detail=True)

    def test_do_goal_show_by_name(self):
        goal = resource.Goal(mock.Mock(), GOAL_1)
        self.m_goal_mgr.get.return_value = goal

        exit_code, result = self.run_cmd('goal show SERVER_CONSOLIDATION')

        self.assertEqual(0, exit_code)
        self.assertEqual(
            self.resource_as_dict(goal, self.FIELDS,
                                  self.FIELD_LABELS),
            result)
        self.m_goal_mgr.get.assert_called_once_with('SERVER_CONSOLIDATION')

    def test_do_goal_show_by_uuid(self):
        goal = resource.Goal(mock.Mock(), GOAL_1)
        self.m_goal_mgr.get.return_value = goal

        exit_code, result = self.run_cmd(
            'goal show fc087747-61be-4aad-8126-b701731ae836')

        self.assertEqual(0, exit_code)
        self.assertEqual(
            self.resource_as_dict(goal, self.FIELDS,
                                  self.FIELD_LABELS),
            result)
        self.m_goal_mgr.get.assert_called_once_with(
            'fc087747-61be-4aad-8126-b701731ae836')



# Copyright 2019 ZTE corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import testtools

from watcherclient.tests.unit import utils
import watcherclient.v1.data_model

DATA_MODEL = {
    'context': [{
        "server_uuid": "1bf91464-9b41-428d-a11e-af691e5563bb",
        "server_name": "fake-name",
        "server_state": "active",
        "node_uuid": "253e5dd0-9384-41ab-af13-4f2c2ce26112",
        "node_hostname": "localhost.localdomain",
        }]
}

AUDIT = "81332bfc-36f8-444d-99e2-b7285d602528"

fake_responses = {
    '/v1/data_model/?data_model_type=compute':
    {
        'GET': (
            {},
            DATA_MODEL,
        ),
    },
    '/v1/data_model/?audit_uuid=%s&data_model_type=compute' % AUDIT:
    {
        'GET': (
            {},
            DATA_MODEL,
        ),
    },
}


class DataModelManagerTest(testtools.TestCase):

    def setUp(self):
        super(DataModelManagerTest, self).setUp()
        self.api = utils.FakeAPI(fake_responses)
        self.mgr = watcherclient.v1.data_model.DataModelManager(self.api)

    def test_data_model_list(self):
        data_model = self.mgr.list()
        expect = [
            ('GET', '/v1/data_model/?data_model_type=compute', {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(1, len(data_model.context))

    def test_data_model_list_audit(self):
        data_model = self.mgr.list(
            audit='%s' % AUDIT)
        expect = [
            ('GET', '/v1/data_model/?'
             'audit_uuid=81332bfc-36f8-444d-99e2-b7285d602528'
             '&data_model_type=compute',
             {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(1, len(data_model.context))



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


# -- General configuration ----------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = ['sphinx.ext.autodoc',
              'sphinx.ext.viewcode',
              'sphinxcontrib.apidoc',
              'openstackdocstheme',
             ]
# autodoc generation is a bit aggressive and a nuisance when doing heavy
# text edit cycles.
# execute "export SPHINX_DEBUG=1" in your terminal to disable

# sphinxcontrib.apidoc options
apidoc_module_dir = '../../watcherclient'
apidoc_output_dir = 'reference/api'
apidoc_excluded_paths = [
    'tests/*']
apidoc_separate_modules = True

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix of source filenames.
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = 'python-watcherclient'
copyright = 'OpenStack Foundation'

# A list of ignored prefixes for module index sorting.
modindex_common_prefix = ['watcherclient.']

# If true, '()' will be appended to :func: etc. cross-reference text.
add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
add_module_names = True

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'native'

# -- Options for HTML output --------------------------------------------------

# The theme to use for HTML and HTML Help pages. Major themes that come with
# Sphinx are currently 'default' and 'sphinxdoc'.
# html_theme_path = ["."]
# html_theme = '_theme'
# html_static_path = ['_static']
html_theme = 'openstackdocs'
# html_theme_path = [openstackdocstheme.get_html_theme_path()]

# Output file base name for HTML help builder.
htmlhelp_basename = '%sdoc' % project


# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title, author, documentclass
# [howto/manual]).
latex_documents = [
    (
        'index',
        '%s.tex' % project,
        '%s Documentation' % project,
        'OpenStack Foundation', 'manual'
    ),
]

# Disable usage of xindy https://bugzilla.redhat.com/show_bug.cgi?id=1643664
latex_use_xindy = False

latex_domain_indices = False

latex_elements = {
    'makeindex': '',
    'printindex': '',
    'preamble': r'\setcounter{tocdepth}{3}',
}

# openstackdocstheme options
openstackdocs_repo_name = 'openstack/python-watcherclient'
openstackdocs_pdf_link = True
openstackdocs_bug_project = 'python-watcherclient'
openstackdocs_bug_tag = ''


#html_theme_options = {"show_other_versions": "True"}



# Copyright 2013 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


import testtools
from testtools import matchers

from watcherclient.tests.unit import utils
import watcherclient.v1.service

SERVICE1 = {
    'id': 1,
    'name': 'watcher-applier',
    'host': 'controller',
    'status': 'ACTIVE',
}

SERVICE2 = {
    'id': 2,
    'name': 'watcher-decision-engine',
    'host': 'controller',
    'status': 'FAILED',
}

fake_responses = {
    '/v1/services':
    {
        'GET': (
            {},
            {"services": [SERVICE1]},
        ),
    },
    '/v1/services/detail':
    {
        'GET': (
            {},
            {"services": [SERVICE1]},
        )
    },
    '/v1/services/%s' % SERVICE1['id']:
    {
        'GET': (
            {},
            SERVICE1,
        ),
    },
    '/v1/services/%s' % SERVICE1['name']:
    {
        'GET': (
            {},
            SERVICE1,
        ),
    },
}

fake_responses_pagination = {
    '/v1/services':
    {
        'GET': (
            {},
            {"services": [SERVICE1],
             "next": "http://127.0.0.1:6385/v1/services/?limit=1"}
        ),
    },
    '/v1/services/?limit=1':
    {
        'GET': (
            {},
            {"services": [SERVICE2]}
        ),
    },
}

fake_responses_sorting = {
    '/v1/services/?sort_key=id':
    {
        'GET': (
            {},
            {"services": [SERVICE1, SERVICE2]}
        ),
    },
    '/v1/services/?sort_dir=desc':
    {
        'GET': (
            {},
            {"services": [SERVICE2, SERVICE1]}
        ),
    },
}


class ServiceManagerTest(testtools.TestCase):

    def setUp(self):
        super(ServiceManagerTest, self).setUp()
        self.api = utils.FakeAPI(fake_responses)
        self.mgr = watcherclient.v1.service.ServiceManager(self.api)

    def test_services_list(self):
        services = self.mgr.list()
        expect = [
            ('GET', '/v1/services', {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(1, len(services))

    def test_services_list_detail(self):
        services = self.mgr.list(detail=True)
        expect = [
            ('GET', '/v1/services/detail', {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(1, len(services))

    def test_services_list_limit(self):
        self.api = utils.FakeAPI(fake_responses_pagination)
        self.mgr = watcherclient.v1.service.ServiceManager(self.api)
        services = self.mgr.list(limit=1)
        expect = [
            ('GET', '/v1/services/?limit=1', {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertThat(services, matchers.HasLength(1))

    def test_services_list_pagination_no_limit(self):
        self.api = utils.FakeAPI(fake_responses_pagination)
        self.mgr = watcherclient.v1.service.ServiceManager(self.api)
        services = self.mgr.list(limit=0)
        expect = [
            ('GET', '/v1/services', {}, None),
            ('GET', '/v1/services/?limit=1', {}, None)
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertThat(services, matchers.HasLength(2))

    def test_services_list_sort_key(self):
        self.api = utils.FakeAPI(fake_responses_sorting)
        self.mgr = watcherclient.v1.service.ServiceManager(self.api)
        services = self.mgr.list(sort_key='id')
        expect = [
            ('GET', '/v1/services/?sort_key=id', {}, None)
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(2, len(services))

    def test_services_list_sort_dir(self):
        self.api = utils.FakeAPI(fake_responses_sorting)
        self.mgr = watcherclient.v1.service.ServiceManager(self.api)
        services = self.mgr.list(sort_dir='desc')
        expect = [
            ('GET', '/v1/services/?sort_dir=desc', {}, None)
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(2, len(services))

    def test_services_show(self):
        service = self.mgr.get(SERVICE1['id'])
        expect = [
            ('GET', '/v1/services/%s' % SERVICE1['id'], {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(SERVICE1['id'], service.id)

    def test_services_show_by_name(self):
        service = self.mgr.get(SERVICE1['name'])
        expect = [
            ('GET', '/v1/services/%s' % SERVICE1['name'], {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertEqual(SERVICE1['name'], service.name)
